var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { dateRangeService } from "../../global/date-range";
import { ActionsGraphClient } from "./actions-graph";
import * as fromActions from "../../reducers/actions-reducer";
import { ApiStoreService } from "../../global/store-provider";
import { ActionsListClient } from "./actions-list-client";
import { ActionsSummaryClient } from "./actions-summary-client";
import { ActionsFilter } from "../../filters/actions-filter";
import { ActionsHeatmapClient } from "./actions-heatmap-client";
import { ActionsIndexAllClient } from "./actions-list-all-client";
import { htClientService } from "../../global/client";
import { distinctUntilChanged, map, filter } from "rxjs/operators";
import { htAction } from "ht-data";
import * as _ from "underscore";
var HtActionsClient = /** @class */ (function () {
    function HtActionsClient(config) {
        this.filters = new ActionsFilter();
        var api = htClientService.getInstance().api.actions;
        this.api = api;
        var store = ApiStoreService.getNewInstance();
        store.addReducer("actions", fromActions.actionsReducer);
        this.store = store;
        var dateRange = config.dateRange;
        var dateParam = 'created_at';
        this.graph = new ActionsGraphClient({ dateRange: dateRange, dateParam: dateParam, api: api, store: store });
        this.list = new ActionsListClient({ dateRange: dateRange, store: store, dateParam: dateParam, api: api });
        this.listAll = new ActionsIndexAllClient({ dateRange: dateRange, dateParam: dateParam, api: api, store: store });
        this.summary = new ActionsSummaryClient({ dateRange: dateRange, store: store, dateParam: dateParam, api: api });
        this.heatmap = new ActionsHeatmapClient({ dateRange: dateRange, dateParam: 'completed_at', api: api, store: store });
        this.initEffects();
    }
    ;
    HtActionsClient.prototype.initEffects = function () {
        var _this = this;
        this.list.query$.pipe(filter(function (data) { return !!data; })).subscribe(function (query) {
            _this.setListAllFilter(query);
        });
        // this.listAll.active$.pipe(filter(data => !!data)).flatMap(() => {
        //   return this.listStatusChart$()
        // })
        //   .takeUntil(this.listAll.active$.filter(data => !data).skip(1))
        //   .withLatestFrom(this.list.query$)
        //   .switchMap(([statusOverview, query]) => {
        //     // return Observable.of({})
        //     console.log(statusOverview, query);
        //     return this.getListAllUpdateQuery$(statusOverview, query)
        //   })//todo finish this
    };
    ;
    HtActionsClient.prototype.setListAllFilter = function (query) {
        var statusString = query["status"];
        var search = query["search"];
        var ids = query["id"];
        var userMarkerFilters = [];
        if (statusString) {
            var statusArray = statusString.split(",");
            // this.updateUserMap(query);
            var statusFilter_1 = [];
            statusArray.forEach(function (status) {
                statusFilter_1.push(htAction().getMarkerFilter(status));
            });
            var allStatusFilter = function (user) {
                return statusFilter_1.reduce(function (acc, filter) {
                    return acc || filter(user);
                }, false);
            };
            userMarkerFilters.push(allStatusFilter);
        }
        if (search) {
            userMarkerFilters.push(function (user) {
                return htAction().getMarkerSearched(search)(user); // || userMarkerFilter(user)
            });
            // this.updateUserMap(query);
        }
        if (ids) {
            var userIds_1 = ids.split(",");
            userMarkerFilters.push(function (user) {
                return userIds_1.includes(user.id);
            });
        }
        var userMarkerFilter = function (user) {
            return userMarkerFilters.reduce(function (acc, filter) {
                return acc && filter(user);
            }, true);
        };
        var dataMap = function (allResults) {
            var results = _.filter(allResults.resultsEntity, userMarkerFilter);
            var resultsEntity = _.indexBy(results, "id");
            return __assign({}, allResults, { resultsEntity: resultsEntity });
        };
        this.listAll.setDataMap(dataMap);
    };
    Object.defineProperty(HtActionsClient.prototype, "queryLabel$", {
        get: function () {
            var _this = this;
            var query$ = this.list.getApiQuery$().pipe(filter(function (data) { return !!data; }));
            return query$.pipe(map(function (query) {
                // console.log("cl", query);
                var queryLabel = _this.filters.getQueryLabel(query);
                return queryLabel;
            }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HtActionsClient.prototype, "ordering$", {
        get: function () {
            var _this = this;
            return this.list.getApiQuery$().pipe(filter(function (data) { return !!data; }), map(function (query) {
                var ordering = query ? query["ordering"] : null;
                var orderingMod = _this.getOrderingMod(ordering);
                return {
                    string: _this.filters.sortingQueryMap[orderingMod.string],
                    sign: orderingMod.sign
                };
            }), distinctUntilChanged());
        },
        enumerable: true,
        configurable: true
    });
    HtActionsClient.prototype.getOrderingMod = function (ordering) {
        var string = ordering;
        var sign = 1;
        if (ordering.includes("-")) {
            string = ordering.substring(1);
            sign = 0;
        }
        return {
            string: string,
            sign: sign
        };
    };
    return HtActionsClient;
}());
export { HtActionsClient };
export var actionsClientFactory = function (options) {
    if (options === void 0) { options = {}; }
    var dateRange = options.dateRange || dateRangeService.getInstance();
    return new HtActionsClient({ dateRange: dateRange });
};
//# sourceMappingURL=actions-client.js.map