import { map, filter, indexBy, values } from 'underscore';
import { timer, combineLatest, of, EMPTY, BehaviorSubject, Observable, Subject, queueScheduler } from 'rxjs';
import { expand, map as map$1, switchMap, distinctUntilChanged, tap, filter as filter$1, pluck, observeOn, scan, withLatestFrom, flatMap, take } from 'rxjs/operators';
import { CombineQueries, AllowedQueryMap, DateRangeToQuery$, PageResults$, itemAsPage$, htUser, DateRangeMap, dateRangeDisplay, CombineLoadings$, htAction } from 'ht-data';
import { Color } from 'ht-utility';
import { HtApi, HtRequest } from 'ht-api';

var defaultListConfig = {
    isLive: false
};
// export interface AllData<T> {
//   resultsEntity: {[id: string]: T},
//   // isFirst: boolean,
//   timestamp?: string,
//   count?: number,
//   next?: string,
//   previous?: string
// }
var ApiType;
(function (ApiType) {
    ApiType["index"] = "index";
    ApiType["analytics"] = "analytics";
})(ApiType || (ApiType = {}));

function combineReducers(reducers, initialState) {
    if (initialState === void 0) { initialState = {}; }
    var reducerKeys = Object.keys(reducers);
    var finalReducers = {};
    for (var i = 0; i < reducerKeys.length; i++) {
        var key = reducerKeys[i];
        if (typeof reducers[key] === "function") {
            finalReducers[key] = reducers[key];
        }
    }
    var finalReducerKeys = Object.keys(finalReducers);
    return function combination(state, action) {
        state = state || initialState;
        var hasChanged = false;
        var nextState = {};
        for (var i = 0; i < finalReducerKeys.length; i++) {
            var key = finalReducerKeys[i];
            var reducer = finalReducers[key];
            var previousStateForKey = state[key];
            var nextStateForKey = reducer(previousStateForKey, action);
            nextState[key] = nextStateForKey;
            hasChanged = hasChanged || nextStateForKey !== previousStateForKey;
        }
        return hasChanged ? nextState : state;
    };
}
function omit(object, keyToRemove) {
    return Object.keys(object)
        .filter(function (key) { return key !== keyToRemove; })
        .reduce(function (result, key) {
        return Object.assign(result, (_a = {}, _a[key] = object[key], _a));
        var _a;
    }, {});
}
function compose() {
    var functions = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        functions[_i] = arguments[_i];
    }
    return function (arg) {
        if (functions.length === 0) {
            return arg;
        }
        var last = functions[functions.length - 1];
        var rest = functions.slice(0, -1);
        return rest.reduceRight(function (composed, fn) { return fn(composed); }, last(arg));
    };
}
function createReducerFactory(reducerFactory, metaReducers) {
    if (Array.isArray(metaReducers) && metaReducers.length > 0) {
        return compose.apply(null, metaReducers.concat([reducerFactory]));
    }
    return reducerFactory;
}

function memoize(t) {
    var lastArguments = null;
    var lastResult = null;
    function reset() {
        lastArguments = null;
        lastResult = null;
    }
    function memoized() {
        if (!lastArguments) {
            lastResult = t.apply(null, arguments);
            lastArguments = arguments;
            return lastResult;
        }
        for (var i = 0; i < arguments.length; i++) {
            if (arguments[i] !== lastArguments[i]) {
                lastResult = t.apply(null, arguments);
                lastArguments = arguments;
                return lastResult;
            }
        }
        return lastResult;
    }
    return { memoized: memoized, reset: reset };
}
function createSelector() {
    var input = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        input[_i] = arguments[_i];
    }
    var args = input;
    if (Array.isArray(args[0])) {
        var head = args[0], tail = args.slice(1);
        args = head.concat(tail);
    }
    var selectors = args.slice(0, args.length - 1);
    var projector = args[args.length - 1];
    var memoizedSelectors = selectors.filter(function (selector) {
        return selector.release && typeof selector.release === "function";
    });
    var memoizedProjector = memoize(function () {
        var selectors = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            selectors[_i] = arguments[_i];
        }
        return projector.apply(null, selectors);
    });
    var memoizedState = memoize(function (state) {
        var args = selectors.map(function (fn) { return fn(state); });
        return memoizedProjector.memoized.apply(null, args);
    });
    function release() {
        memoizedState.reset();
        memoizedProjector.reset();
        memoizedSelectors.forEach(function (selector) { return selector.release(); });
    }
    return Object.assign(memoizedState.memoized, {
        release: release,
        projector: memoizedProjector.memoized
    });
}
function createFeatureSelector(featureName) {
    return createSelector(function (state) { return state[featureName]; }, function (featureState) { return featureState; });
}

//placeline
var SET_USER_DATA = "[USERS] set user data";
var SET_PLACELINE_ID = "[USERS] set placeline id";
var TOGGLE_PLACELINE_ID = "[USERS] toggle placeline id";
var SET_PLACELINE_QUERY = "[USERS] set placeline query";
var ADD_PLACELINE_QUERY = "[USERS] add placeline query";
var SET_PLACELINE_LOADING = "[USERS] set placeline loading";
//list
var SET_LIST_ACTIVE = "[USERS] set list active";
var SET_LIST_ID = "[USERS] set list id";
var SET_LIST_QUERY = "[USERS] set list query";
var ADD_LIST_QUERY = "[USERS] add list query";
var CLEAR_QUERY_KEY = "[USERS] clear query key";
var TOGGLE_LIST_ID = "[USERS] toggle list id";
//analytics page
var SET_USERS_ANALYTICS_PAGE = "[USERS] set user analytics page";
var SET_USERS_ANALYTICS_ID = "[USERS] set analytics id";
var SET_USERS_ANALYTICS_QUERY = "[USERS] set analytics query";
var SET_USERS_ANALYTICS_LOADING = "[USERS] set analytics loading";
var SET_USERS_LIST_DATA_MAP = "[USERS] set users list data map";
//analytics all
var SET_MARKERS_ACTIVE = "[USERS] set markerS active";
var SET_USERS_ANALYTICS_ALL = "[USERS] set users analytics all";
var ADD_USERS_ANALYTICS_ALL = "[USERS] add users analytics all";
var SET_USERS_ANALYTICS_ALL_QUERY = "[USERS] add analytics all query";
var ADD_USERS_ANALYTICS_ALL_QUERY = "[USERS] set analytics all query";
var SET_USERS_ANALYTICS_ALL_LOADING = "[USERS] set analytics all loading";
var SET_USERS_MARKERS_DATA_MAP = "[USERS] set users listAll data map";
//summary
var SET_SUMMARY_ACTIVE = "[USERS] set summary active";
var SET_USERS_SUMMARY = "[USERS] set users summary";
var SET_USERS_SUMMARY_LOADING = "[USERS] set summary loading";
var SET_USERS_INDEX_PAGE = "[USERS] set user index page";
var ADD_USERS_INDEX_ALL = "[USERS] add users index all";
var SET_USERS_LIST_API_TYPE = "[USERS] set users list api type";
var SetUserData = /** @class */ (function () {
    function SetUserData(payload) {
        this.payload = payload;
        this.type = SET_USER_DATA;
    }
    return SetUserData;
}());
var SetPlacelineId = /** @class */ (function () {
    function SetPlacelineId(payload) {
        this.payload = payload;
        this.type = SET_PLACELINE_ID;
    }
    return SetPlacelineId;
}());
var TogglePlacelineId = /** @class */ (function () {
    function TogglePlacelineId(payload) {
        this.payload = payload;
        this.type = TOGGLE_PLACELINE_ID;
    }
    return TogglePlacelineId;
}());
var SetPlacelineQuery = /** @class */ (function () {
    function SetPlacelineQuery(payload) {
        this.payload = payload;
        this.type = SET_PLACELINE_QUERY;
    }
    return SetPlacelineQuery;
}());
var AddPlacelineQuery = /** @class */ (function () {
    function AddPlacelineQuery(payload) {
        this.payload = payload;
        this.type = ADD_PLACELINE_QUERY;
    }
    return AddPlacelineQuery;
}());
var SetPlacelineLoading = /** @class */ (function () {
    function SetPlacelineLoading(payload) {
        this.payload = payload;
        this.type = SET_PLACELINE_LOADING;
    }
    return SetPlacelineLoading;
}());
var SetUsersListId = /** @class */ (function () {
    function SetUsersListId(payload) {
        this.payload = payload;
        this.type = SET_LIST_ID;
    }
    return SetUsersListId;
}());
var ToggleUsersListId = /** @class */ (function () {
    function ToggleUsersListId(payload) {
        this.payload = payload;
        this.type = TOGGLE_LIST_ID;
    }
    return ToggleUsersListId;
}());
var SetListQuery = /** @class */ (function () {
    function SetListQuery(payload) {
        this.payload = payload;
        this.type = SET_LIST_QUERY;
    }
    return SetListQuery;
}());
var AddListQuery = /** @class */ (function () {
    function AddListQuery(payload) {
        this.payload = payload;
        this.type = ADD_LIST_QUERY;
    }
    return AddListQuery;
}());
var ClearQueryKey = /** @class */ (function () {
    function ClearQueryKey(payload) {
        this.payload = payload;
        this.type = CLEAR_QUERY_KEY;
    }
    return ClearQueryKey;
}());
var SetUsersAnalyticsPage = /** @class */ (function () {
    function SetUsersAnalyticsPage(payload) {
        this.payload = payload;
        this.type = SET_USERS_ANALYTICS_PAGE;
    }
    return SetUsersAnalyticsPage;
}());
var SetUsersAnalyticsLoading = /** @class */ (function () {
    function SetUsersAnalyticsLoading(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_USERS_ANALYTICS_LOADING;
    }
    return SetUsersAnalyticsLoading;
}());
var SetUsersIndexPage = /** @class */ (function () {
    function SetUsersIndexPage(payload) {
        this.payload = payload;
        this.type = SET_USERS_INDEX_PAGE;
    }
    return SetUsersIndexPage;
}());
var AddUsersAnalyticsAll = /** @class */ (function () {
    function AddUsersAnalyticsAll(payload) {
        this.payload = payload;
        this.type = ADD_USERS_ANALYTICS_ALL;
    }
    return AddUsersAnalyticsAll;
}());
var SetUsersAnalyticsAll = /** @class */ (function () {
    function SetUsersAnalyticsAll(payload) {
        this.payload = payload;
        this.type = SET_USERS_ANALYTICS_ALL;
    }
    return SetUsersAnalyticsAll;
}());
var SetUsersAnalyticsAllLoading = /** @class */ (function () {
    function SetUsersAnalyticsAllLoading(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_USERS_ANALYTICS_ALL_LOADING;
    }
    return SetUsersAnalyticsAllLoading;
}());
var SetUsersIndexAll = /** @class */ (function () {
    function SetUsersIndexAll(payload) {
        this.payload = payload;
        //todo change name
        this.type = ADD_USERS_INDEX_ALL;
    }
    return SetUsersIndexAll;
}());
var SetUsersListApiType = /** @class */ (function () {
    function SetUsersListApiType(payload) {
        this.payload = payload;
        this.type = SET_USERS_LIST_API_TYPE;
    }
    return SetUsersListApiType;
}());
var SetUsersListDataMap = /** @class */ (function () {
    function SetUsersListDataMap(payload) {
        this.payload = payload;
        this.type = SET_USERS_LIST_DATA_MAP;
    }
    return SetUsersListDataMap;
}());
var SetUsersMarkersDataMap = /** @class */ (function () {
    function SetUsersMarkersDataMap(payload) {
        this.payload = payload;
        this.type = SET_USERS_MARKERS_DATA_MAP;
    }
    return SetUsersMarkersDataMap;
}());
var SetListActive = /** @class */ (function () {
    function SetListActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_LIST_ACTIVE;
    }
    return SetListActive;
}());
var SetMarkersActive = /** @class */ (function () {
    function SetMarkersActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_MARKERS_ACTIVE;
    }
    return SetMarkersActive;
}());
var SetSummaryActive = /** @class */ (function () {
    function SetSummaryActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_SUMMARY_ACTIVE;
    }
    return SetSummaryActive;
}());
var SetUsersSummary = /** @class */ (function () {
    function SetUsersSummary(payload) {
        this.payload = payload;
        this.type = SET_USERS_SUMMARY;
    }
    return SetUsersSummary;
}());
var SetSummaryLoading = /** @class */ (function () {
    function SetSummaryLoading(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_USERS_SUMMARY_LOADING;
    }
    return SetSummaryLoading;
}());

var UsersDispatcher = /*#__PURE__*/Object.freeze({
    SET_USER_DATA: SET_USER_DATA,
    SET_PLACELINE_ID: SET_PLACELINE_ID,
    TOGGLE_PLACELINE_ID: TOGGLE_PLACELINE_ID,
    SET_PLACELINE_QUERY: SET_PLACELINE_QUERY,
    ADD_PLACELINE_QUERY: ADD_PLACELINE_QUERY,
    SET_PLACELINE_LOADING: SET_PLACELINE_LOADING,
    SET_LIST_ACTIVE: SET_LIST_ACTIVE,
    SET_LIST_ID: SET_LIST_ID,
    SET_LIST_QUERY: SET_LIST_QUERY,
    ADD_LIST_QUERY: ADD_LIST_QUERY,
    CLEAR_QUERY_KEY: CLEAR_QUERY_KEY,
    TOGGLE_LIST_ID: TOGGLE_LIST_ID,
    SET_USERS_ANALYTICS_PAGE: SET_USERS_ANALYTICS_PAGE,
    SET_USERS_ANALYTICS_ID: SET_USERS_ANALYTICS_ID,
    SET_USERS_ANALYTICS_QUERY: SET_USERS_ANALYTICS_QUERY,
    SET_USERS_ANALYTICS_LOADING: SET_USERS_ANALYTICS_LOADING,
    SET_USERS_LIST_DATA_MAP: SET_USERS_LIST_DATA_MAP,
    SET_MARKERS_ACTIVE: SET_MARKERS_ACTIVE,
    SET_USERS_ANALYTICS_ALL: SET_USERS_ANALYTICS_ALL,
    ADD_USERS_ANALYTICS_ALL: ADD_USERS_ANALYTICS_ALL,
    SET_USERS_ANALYTICS_ALL_QUERY: SET_USERS_ANALYTICS_ALL_QUERY,
    ADD_USERS_ANALYTICS_ALL_QUERY: ADD_USERS_ANALYTICS_ALL_QUERY,
    SET_USERS_ANALYTICS_ALL_LOADING: SET_USERS_ANALYTICS_ALL_LOADING,
    SET_USERS_MARKERS_DATA_MAP: SET_USERS_MARKERS_DATA_MAP,
    SET_SUMMARY_ACTIVE: SET_SUMMARY_ACTIVE,
    SET_USERS_SUMMARY: SET_USERS_SUMMARY,
    SET_USERS_SUMMARY_LOADING: SET_USERS_SUMMARY_LOADING,
    SET_USERS_INDEX_PAGE: SET_USERS_INDEX_PAGE,
    ADD_USERS_INDEX_ALL: ADD_USERS_INDEX_ALL,
    SET_USERS_LIST_API_TYPE: SET_USERS_LIST_API_TYPE,
    SetUserData: SetUserData,
    SetPlacelineId: SetPlacelineId,
    TogglePlacelineId: TogglePlacelineId,
    SetPlacelineQuery: SetPlacelineQuery,
    AddPlacelineQuery: AddPlacelineQuery,
    SetPlacelineLoading: SetPlacelineLoading,
    SetUsersListId: SetUsersListId,
    ToggleUsersListId: ToggleUsersListId,
    SetListQuery: SetListQuery,
    AddListQuery: AddListQuery,
    ClearQueryKey: ClearQueryKey,
    SetUsersAnalyticsPage: SetUsersAnalyticsPage,
    SetUsersAnalyticsLoading: SetUsersAnalyticsLoading,
    SetUsersIndexPage: SetUsersIndexPage,
    AddUsersAnalyticsAll: AddUsersAnalyticsAll,
    SetUsersAnalyticsAll: SetUsersAnalyticsAll,
    SetUsersAnalyticsAllLoading: SetUsersAnalyticsAllLoading,
    SetUsersIndexAll: SetUsersIndexAll,
    SetUsersListApiType: SetUsersListApiType,
    SetUsersListDataMap: SetUsersListDataMap,
    SetUsersMarkersDataMap: SetUsersMarkersDataMap,
    SetListActive: SetListActive,
    SetMarkersActive: SetMarkersActive,
    SetSummaryActive: SetSummaryActive,
    SetUsersSummary: SetUsersSummary,
    SetSummaryLoading: SetSummaryLoading
});

var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var initialUsersAnalyticsAll = {
    resultsEntity: {},
    isFirst: false
};
var initialUsersIndexAll = {
    resultsEntity: {},
    isFirst: false
};
var initialState = {
    usersListActive: false,
    usersMarkersActive: false,
    usersSummaryActive: false,
    listApiType: ApiType.analytics,
    usersAnalyticsAll: initialUsersAnalyticsAll,
    usersIndexAll: initialUsersIndexAll
};
function usersReducer(state, action) {
    if (state === void 0) { state = initialState; }
    switch (action.type) {
        /*
        placeline
         */
        case SET_USER_DATA: {
            return __assign({}, state, { userData: action.payload });
        }
        case SET_PLACELINE_ID: {
            return __assign({}, state, { placelineId: action.payload });
        }
        case TOGGLE_PLACELINE_ID: {
            var placelineId = state.placelineId == action.payload ? null : action.payload;
            return __assign({}, state, { placelineId: placelineId });
        }
        case SET_PLACELINE_QUERY: {
            return __assign({}, state, { placelineQuery: action.payload });
        }
        case ADD_PLACELINE_QUERY: {
            return __assign({}, state, { placelineQuery: __assign({}, state.placelineQuery, action.payload) });
        }
        case SET_PLACELINE_LOADING: {
            return __assign({}, state, { placelineLoading: !!action.payload });
        }
        /*
        List
         */
        case SET_LIST_ID: {
            return __assign({}, state, { listId: action.payload });
        }
        case TOGGLE_LIST_ID: {
            var listId = state.listId == action.payload ? null : action.payload;
            return __assign({}, state, { listId: listId });
        }
        case SET_LIST_ACTIVE: {
            return __assign({}, state, { usersListActive: action.payload });
        }
        case ADD_LIST_QUERY: {
            return __assign({}, state, { listQuery: __assign({}, state.listQuery, action.payload) });
        }
        case SET_LIST_QUERY: {
            return __assign({}, state, { listQuery: action.payload });
        }
        case CLEAR_QUERY_KEY: {
            var listQuery = __assign({}, state.listQuery);
            if (listQuery) {
                delete listQuery[action.payload];
                delete listQuery['page'];
            }
            return __assign({}, state, { listQuery: listQuery });
        }
        /*
        Analytics page
         */
        case SET_USERS_ANALYTICS_PAGE: {
            return __assign({}, state, { usersAnalyticsPage: action.payload });
        }
        case SET_USERS_ANALYTICS_LOADING: {
            return __assign({}, state, { analyticsLoading: action.payload });
        }
        case SET_USERS_INDEX_PAGE: {
            return __assign({}, state, { usersIndexPage: action.payload });
        }
        case ADD_USERS_ANALYTICS_ALL: {
            var resultsEntity = {};
            if (state.usersAnalyticsAll) {
                resultsEntity = __assign({}, state.usersAnalyticsAll.resultsEntity, indexBy(action.payload.results, "id"));
            }
            return __assign({}, state, { usersAnalyticsAll: {
                    next: action.payload.next,
                    previous: action.payload.previous,
                    count: action.payload.count,
                    resultsEntity: resultsEntity
                } });
        }
        case SET_USERS_ANALYTICS_ALL: {
            var resultsEntity = {};
            if (state.usersAnalyticsAll) {
                resultsEntity = indexBy(action.payload.results, "id");
            }
            return __assign({}, state, { usersAnalyticsAll: {
                    next: action.payload.next,
                    previous: action.payload.previous,
                    count: action.payload.count,
                    resultsEntity: resultsEntity
                } });
        }
        case SET_USERS_ANALYTICS_ALL_LOADING: {
            return __assign({}, state, { analyticsAllLoading: action.payload });
        }
        case ADD_USERS_INDEX_ALL: {
            // const newEntities = _.indexBy(action.payload.results, 'id');
            var resultsEntity = state.usersIndexAll
                ? __assign({}, state.usersIndexAll.resultsEntity, action.payload.resultsEntity) : {};
            return __assign({}, state, { usersIndexAll: __assign({}, action.payload, { resultsEntity: resultsEntity }) });
        }
        case SET_USERS_LIST_DATA_MAP: {
            return __assign({}, state, { usersListDataMap: action.payload });
        }
        case SET_USERS_MARKERS_DATA_MAP: {
            return __assign({}, state, { usersMarkersDataMap: action.payload });
        }
        case SET_USERS_LIST_API_TYPE: {
            return __assign({}, state, { listApiType: action.payload });
        }
        case SET_MARKERS_ACTIVE: {
            return __assign({}, state, { usersMarkersActive: action.payload });
        }
        /*
        Summary
         */
        case SET_SUMMARY_ACTIVE: {
            return __assign({}, state, { usersSummaryActive: action.payload });
        }
        case SET_USERS_SUMMARY: {
            return __assign({}, state, { usersSummary: action.payload });
        }
        case SET_USERS_SUMMARY_LOADING: {
            return __assign({}, state, { summaryLoading: !!action.payload });
        }
        default: {
            return state;
        }
    }
}
var getUserData = function (state) { return state.userData; };
var getPlacelineId = function (state) { return state.placelineId; };
var getPlacelineQuery = function (state) { return state.placelineQuery; };
var getPlacelineLoading = function (state) { return state.placelineLoading; };
var getListActive = function (state) { return state.usersListActive; };
var getListId = function (state) { return state.listId; };
var getListQuery = function (state) { return state.listQuery; };
var getAnalyticsPage = function (state) { return state.usersAnalyticsPage; };
var getAnalyticsLoading = function (state) { return state.analyticsLoading; };
var getAnalyticsAll = function (state) { return state.usersAnalyticsAll; };
var getAnalyticsAllLoading = function (state) {
    return state.analyticsAllLoading;
};
var getMarkerDataMap = function (state) { return state.usersMarkersDataMap; };
var getIndexAll = function (state) { return state.usersIndexAll; };
// export const getAnalyticFilteredsMarkers = (state: State) => validMarkers(state.usersAnalyticsAll);
// export const getIndexFilteredMarkers = (state: State) => validMarkers(state.usersIndexAll);
var getIndexPage = function (state) { return state.usersIndexPage; };
var getListApiType = function (state) { return state.listApiType; };
var getMarkersActive = function (state) { return state.usersMarkersActive; };
var getSummary = function (state) { return state.usersSummary; };
var getSummaryActive = function (state) { return state.usersSummaryActive; };
var getSummaryLoading = function (state) { return state.summaryLoading; };
var getAnalyticFilteredMarkers = createSelector(getAnalyticsAll, getMarkerDataMap, function (allData, mapFunc) {
    return mapFunc ? mapFunc(allData) : allData;
});
var getIndexFilteredMarkers = createSelector(getIndexAll, getMarkerDataMap, function (allData, mapFunc) {
    return mapFunc ? mapFunc(allData) : allData;
});
var getIndexActive = createSelector(getListApiType, getListActive, function (apiType, isListActive) {
    return false;
    // return apiType === ApiType.index && isListActive
});
var getAnalyticsActive = createSelector(getListApiType, getListActive, function (apiType, isListActive) {
    return isListActive;
    // return apiType === ApiType.analytics && isListActive
});
var getIndexMarkersActive = createSelector(getListApiType, getMarkersActive, function (apiType, isMarkersActive) {
    return false;
    // return apiType === ApiType.index && isMarkersActive
});
var getAnalyticsMarkersActive = createSelector(getListApiType, getMarkersActive, function (apiType, isMarkersActive) {
    return isMarkersActive;
    // return apiType === ApiType.analytics && isMarkersActive
});

var SET_SELECTED_ID = "[SEGMENTS] set selected id";
var SET_HIGHLIGHTED_ID = "[SEGMENTS] set highlighted id";
var SET_RESET_MAP_ID = "[SEGMENTS] set reset map id";
var SET_POPUP_ID = "[SEGMENTS] set popup id";
var SetSelectedId = /** @class */ (function () {
    function SetSelectedId(payload) {
        this.payload = payload;
        this.type = SET_SELECTED_ID;
    }
    return SetSelectedId;
}());
var SetHighlightedId = /** @class */ (function () {
    function SetHighlightedId(payload) {
        this.payload = payload;
        this.type = SET_HIGHLIGHTED_ID;
    }
    return SetHighlightedId;
}());
var SetResetMapId = /** @class */ (function () {
    function SetResetMapId(payload) {
        this.payload = payload;
        this.type = SET_RESET_MAP_ID;
    }
    return SetResetMapId;
}());
var SetPopupId = /** @class */ (function () {
    function SetPopupId(payload) {
        this.payload = payload;
        this.type = SET_POPUP_ID;
    }
    return SetPopupId;
}());

var SegmentsDispatcher = /*#__PURE__*/Object.freeze({
    SET_SELECTED_ID: SET_SELECTED_ID,
    SET_HIGHLIGHTED_ID: SET_HIGHLIGHTED_ID,
    SET_RESET_MAP_ID: SET_RESET_MAP_ID,
    SET_POPUP_ID: SET_POPUP_ID,
    SetSelectedId: SetSelectedId,
    SetHighlightedId: SetHighlightedId,
    SetResetMapId: SetResetMapId,
    SetPopupId: SetPopupId
});

var __assign$1 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var initialState$1 = {};
function segmentsReducer(state, action) {
    if (state === void 0) { state = initialState$1; }
    switch (action.type) {
        case SET_SELECTED_ID: {
            return __assign$1({}, state, { selectedId: action.payload });
        }
        case SET_RESET_MAP_ID: {
            return __assign$1({}, state, { resetMapId: action.payload });
        }
        case SET_HIGHLIGHTED_ID: {
            return !!state.selectedId
                ? __assign$1({}, state, { highlightedId: action.payload }) : state;
        }
        case SET_POPUP_ID: {
            return !!state.selectedId ? __assign$1({}, state, { popupId: action.payload }) : state;
        }
        default: {
            return state;
        }
    }
}
var getSelectedId = function (state) { return state.selectedId; };
var getResetMapId = function (state) { return state.resetMapId; };

var SET_LIST_ACTIVE$1 = "[GROUPS] set list active";
var SET_ID = "[GROUPS] set id";
var SET_GROUPS = "[GROUPS] set groups";
var SET_GROUP = "[GROUPS] set group";
var SetListActive$1 = /** @class */ (function () {
    function SetListActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_LIST_ACTIVE$1;
    }
    return SetListActive;
}());
var SetGroupId = /** @class */ (function () {
    function SetGroupId(payload) {
        this.payload = payload;
        this.type = SET_ID;
    }
    return SetGroupId;
}());
var SetGroupsAll = /** @class */ (function () {
    function SetGroupsAll(payload) {
        this.payload = payload;
        this.type = SET_GROUPS;
    }
    return SetGroupsAll;
}());
var SetGroup = /** @class */ (function () {
    function SetGroup(payload) {
        this.payload = payload;
        this.type = SET_GROUP;
    }
    return SetGroup;
}());

var GroupsDispatcher = /*#__PURE__*/Object.freeze({
    SET_LIST_ACTIVE: SET_LIST_ACTIVE$1,
    SET_ID: SET_ID,
    SET_GROUPS: SET_GROUPS,
    SET_GROUP: SET_GROUP,
    SetListActive: SetListActive$1,
    SetGroupId: SetGroupId,
    SetGroupsAll: SetGroupsAll,
    SetGroup: SetGroup
});

var __assign$2 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var initialState$2 = {
    listActive: false
};
function groupsReducer(state, action) {
    if (state === void 0) { state = initialState$2; }
    switch (action.type) {
        case SET_LIST_ACTIVE$1: {
            return __assign$2({}, state, { listActive: action.payload });
        }
        case SET_ID: {
            return __assign$2({}, state, { selectedId: action.payload });
        }
        case SET_GROUPS: {
            return __assign$2({}, state, { allGroups: action.payload });
        }
        case SET_GROUP: {
            return __assign$2({}, state, { group: action.payload });
        }
        default: {
            return state;
        }
    }
}
var getId = function (state) { return state.selectedId; };
var getAllGroups = function (state) { return state.allGroups; };
var getListActive$1 = function (state) { return state.listActive; };

var SET_ACCOUNT_USER = "[ACCOUNT] set account user";
var SET_MEMBERHSHIPS_ALL = "[ACCOUNT] set memberships all";
var SET_KEY = "[ACCOUNT] set key";
var SET_TEMP_KEY = "[ACCOUNT] set temp key";
var SET_USER_ID = "[ACCOUNT] set user id";
var SetAccountUser = /** @class */ (function () {
    function SetAccountUser(payload) {
        this.payload = payload;
        this.type = SET_ACCOUNT_USER;
    }
    return SetAccountUser;
}());
var SetMembershipsAll = /** @class */ (function () {
    function SetMembershipsAll(payload) {
        this.payload = payload;
        this.type = SET_MEMBERHSHIPS_ALL;
    }
    return SetMembershipsAll;
}());
var SetKey = /** @class */ (function () {
    function SetKey(payload) {
        this.payload = payload;
        this.type = SET_KEY;
    }
    return SetKey;
}());
var SetUserId = /** @class */ (function () {
    function SetUserId(payload) {
        this.payload = payload;
        this.type = SET_USER_ID;
    }
    return SetUserId;
}());

var __assign$3 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var initialState$3 = {
    key: "false"
};
function reducer(state, action) {
    if (state === void 0) { state = initialState$3; }
    switch (action.type) {
        case SET_ACCOUNT_USER: {
            return __assign$3({}, state, { accountUser: action.payload });
        }
        case SET_MEMBERHSHIPS_ALL: {
            // let results = state.membershipsAll ? [...state.membershipsAll.results, ...action.payload.results] : action.payload.results;
            // return {...state, membershipsAll: {...action.payload, results}}
            return __assign$3({}, state, { membershipsAll: action.payload });
        }
        case SET_KEY: {
            return __assign$3({}, state, { key: action.payload });
        }
        case SET_TEMP_KEY: {
            return __assign$3({}, state, { tempKey: action.payload });
        }
        case SET_USER_ID: {
            return __assign$3({}, state, { userId: action.payload });
        }
        default: {
            return state;
        }
    }
}
var getAccountUser = function (state) { return state.accountUser; };
var getMembershipsAll = function (state) { return state.membershipsAll; };
var getKey = function (state) { return state.key; };
var getTempKey = function (state) { return state.tempKey; };
var getUserId = function (state) { return state.userId; };
var getCurrentKey = createSelector(getKey, getTempKey, function (key, tempKey) {
    return tempKey || key;
});

/*
List
 */
var SET_ACTIONS_LIST = "[ACTIONS] set actions list";
var SET_ACTIONS_LIST_QUERY = "[ACTIONS] set actions list query";
var CLEAR_ACTION_QUERY_KEY = "[ACTIONS] clear list query key";
var ADD_ACTIONS_LIST_QUERY = "[ACTIONS] add actions list query";
var SET_ACTIONS_LIST_ACTIVE = "[ACTIONS] set actions list active";
var SET_ACTIONS_LIST_LOADING = "[ACTIONS] set actions list loading";
/*
List all
 */
var SET_ACTIONS_LIST_ALL = "[ACTIONS] set actions list all";
var ADD_ACTIONS_LIST_ALL = "[ACTIONS] add actions list all";
var SET_ACTIONS_LIST_ALL_ACTIVE = "[ACTIONS] set actions list all active";
var SET_ACTIONS_LIST_ALL_LOADING = "[ACTIONS] set actions list all loading";
var SET_ACTIONS_LIST_ALL_DATA_MAP = "[ACTIONS] set actions list all data map";
/*
Graph
 */
var SET_ACTIONS_GRAPH = "[ACTIONS] set actions graph";
var SET_ACTIONS_GRAPH_QUERY = "[ACTIONS] set actions graph query";
var ADD_ACTIONS_GRAPH_QUERY = "[ACTIONS] add actions graph query";
var SET_ACTIONS_GRAPH_LOADING = "[ACTIONS] set actions graph loading";
var SET_ACTIONS_GRAPH_ACTIVE = "[ACTIONS] set actions graph active";
/*
Summary
 */
var SET_ACTIONS_SUMMARY = "[ACTIONS] set actions summary";
var SET_ACTIONS_SUMMARY_ACTIVE = "[ACTIONS] set actions summary active";
var ADD_ACTIONS_SUMMARY_QUERY = "[ACTIONS] actions summary query";
var SET_ACTIONS_SUMMARY_QUERY = "[ACTIONS] add set actions summary query";
var SET_ACTIONS_SUMMARY_LOADING = "[ACTIONS] set actions summary loading";
/*
Heatmap
 */
// export const SET_ACTIONS_HEATMAP = "[ACTIONS] set actions summary";
// export const SET_ACTIONS_HEATMAP_ACTIVE = "[ACTIONS] set actions summary active";
// export const ADD_ACTIONS_HEATMAP_QUERY = "[ACTIONS] actions summary query";
// export const SET_ACTIONS_HEATMAP_QUERY = "[ACTIONS] add set actions summary query";
// export const SET_ACTIONS_HEATMAP_LOADING = "[ACTIONS] set actions graph loading";
var SetList = /** @class */ (function () {
    function SetList(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_LIST;
    }
    return SetList;
}());
var SetListQuery$1 = /** @class */ (function () {
    function SetListQuery(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_QUERY;
    }
    return SetListQuery;
}());
var AddListQuery$1 = /** @class */ (function () {
    function AddListQuery(payload) {
        this.payload = payload;
        this.type = ADD_ACTIONS_LIST_QUERY;
    }
    return AddListQuery;
}());
var ClearQueryKey$1 = /** @class */ (function () {
    function ClearQueryKey(payload) {
        this.payload = payload;
        this.type = CLEAR_ACTION_QUERY_KEY;
    }
    return ClearQueryKey;
}());
var SetListActive$2 = /** @class */ (function () {
    function SetListActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_ACTIVE;
    }
    return SetListActive;
}());
var SetListLoading = /** @class */ (function () {
    function SetListLoading(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_LOADING;
    }
    return SetListLoading;
}());
var SetListAll = /** @class */ (function () {
    function SetListAll(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_ALL;
    }
    return SetListAll;
}());
var AddListAll = /** @class */ (function () {
    function AddListAll(payload) {
        this.payload = payload;
        this.type = ADD_ACTIONS_LIST_ALL;
    }
    return AddListAll;
}());
var SetListAllLoading = /** @class */ (function () {
    function SetListAllLoading(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_ALL_LOADING;
    }
    return SetListAllLoading;
}());
var SetListAllActive = /** @class */ (function () {
    function SetListAllActive(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_ALL_ACTIVE;
    }
    return SetListAllActive;
}());
var SetListAllDataMap = /** @class */ (function () {
    function SetListAllDataMap(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_LIST_ALL_DATA_MAP;
    }
    return SetListAllDataMap;
}());
var SetGraph = /** @class */ (function () {
    function SetGraph(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_GRAPH;
    }
    return SetGraph;
}());
var SetGraphLoading = /** @class */ (function () {
    function SetGraphLoading(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_ACTIONS_GRAPH_LOADING;
    }
    return SetGraphLoading;
}());
var SetGraphActive = /** @class */ (function () {
    function SetGraphActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_ACTIONS_GRAPH_ACTIVE;
    }
    return SetGraphActive;
}());
var SetSummary = /** @class */ (function () {
    function SetSummary(payload) {
        this.payload = payload;
        this.type = SET_ACTIONS_SUMMARY;
    }
    return SetSummary;
}());
var SetSummaryLoading$1 = /** @class */ (function () {
    function SetSummaryLoading(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_ACTIONS_SUMMARY_LOADING;
    }
    return SetSummaryLoading;
}());
var SetSummaryActive$1 = /** @class */ (function () {
    function SetSummaryActive(payload) {
        if (payload === void 0) { payload = true; }
        this.payload = payload;
        this.type = SET_ACTIONS_SUMMARY_ACTIVE;
    }
    return SetSummaryActive;
}());

var __assign$4 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var initialState$4 = {
    list: null,
    listLoading: false,
    listQuery: {},
    listActive: false,
    listAll: null,
    listAllLoading: false,
    listAllActive: false,
    summary: null,
    summaryActive: false,
    summaryLoading: false,
    summaryQuery: {},
    graph: null,
    graphLoading: false,
    graphQuery: {},
    graphActive: false
};
function actionsReducer(state, action) {
    if (state === void 0) { state = initialState$4; }
    switch (action.type) {
        /*
        list
         */
        case SET_ACTIONS_LIST: {
            return __assign$4({}, state, { list: action.payload });
        }
        case SET_ACTIONS_LIST_ACTIVE: {
            return __assign$4({}, state, { listActive: action.payload });
        }
        case SET_ACTIONS_LIST_QUERY: {
            return __assign$4({}, state, { listQuery: action.payload });
        }
        case CLEAR_ACTION_QUERY_KEY: {
            var listQuery = __assign$4({}, state.listQuery);
            if (listQuery) {
                delete listQuery[action.payload];
                delete listQuery['page'];
            }
            return __assign$4({}, state, { listQuery: listQuery });
        }
        case ADD_ACTIONS_LIST_QUERY: {
            var listQuery = __assign$4({}, state.listQuery, action.payload);
            return __assign$4({}, state, { listQuery: listQuery });
        }
        case SET_ACTIONS_LIST_LOADING: {
            return __assign$4({}, state, { listLoading: action.payload });
        }
        /*
        List all
         */
        case SET_ACTIONS_LIST_ALL: {
            var listAllPage = action.payload;
            if (listAllPage) {
                return __assign$4({}, state, { listAll: {
                        count: listAllPage.count,
                        next: listAllPage.next,
                        previous: listAllPage.previous,
                        resultsEntity: indexBy(listAllPage.results, 'id')
                    } });
            }
            else {
                return __assign$4({}, state, { listAll: null });
            }
        }
        case ADD_ACTIONS_LIST_ALL: {
            var listAllPage = action.payload;
            var currentResultsEntities = state.listAll ? state.listAll.resultsEntity : {};
            var resultsEntity = __assign$4({}, currentResultsEntities, indexBy(listAllPage.results, 'id'));
            return __assign$4({}, state, { listAll: {
                    count: listAllPage.count,
                    next: listAllPage.next,
                    previous: listAllPage.previous,
                    resultsEntity: resultsEntity
                } });
        }
        case SET_ACTIONS_LIST_ALL_ACTIVE: {
            return __assign$4({}, state, { listAllActive: action.payload });
        }
        case SET_ACTIONS_LIST_ALL_LOADING: {
            return __assign$4({}, state, { listAllLoading: action.payload });
        }
        case SET_ACTIONS_LIST_ALL_DATA_MAP: {
            return __assign$4({}, state, { listAllDataMap: action.payload });
        }
        /*
        Summary
         */
        case SET_ACTIONS_SUMMARY: {
            return __assign$4({}, state, { summary: action.payload });
        }
        case SET_ACTIONS_SUMMARY_LOADING: {
            return __assign$4({}, state, { summaryLoading: action.payload });
        }
        case SET_ACTIONS_SUMMARY_ACTIVE: {
            return __assign$4({}, state, { summaryActive: action.payload });
        }
        case SET_ACTIONS_SUMMARY_QUERY: {
            return __assign$4({}, state, { summaryQuery: action.payload });
        }
        case ADD_ACTIONS_SUMMARY_QUERY: {
            var summaryQuery = __assign$4({}, state.summaryQuery, action.payload);
            return __assign$4({}, state, { summaryQuery: summaryQuery });
        }
        /*
        Graph
         */
        case SET_ACTIONS_GRAPH: {
            return __assign$4({}, state, { graph: action.payload });
        }
        case SET_ACTIONS_GRAPH_LOADING: {
            return __assign$4({}, state, { graphLoading: action.payload });
        }
        case SET_ACTIONS_GRAPH_QUERY: {
            return __assign$4({}, state, { graphQuery: action.payload });
        }
        case ADD_ACTIONS_GRAPH_QUERY: {
            var graphQuery = __assign$4({}, state.graphQuery, action.payload);
            return __assign$4({}, state, { graphQuery: graphQuery });
        }
        case SET_ACTIONS_GRAPH_ACTIVE: {
            return __assign$4({}, state, { graphActive: action.payload });
        }
        default: {
            return state;
        }
    }
}
var getList = function (state) { return state.list; };
var getListLoading = function (state) { return state.listLoading; };
var getListActive$2 = function (state) { return state.listActive; };
var getListQuery$1 = function (state) { return state.listQuery; };
var getListAll = function (state) { return state.listAll; };
var getListAllLoading = function (state) { return state.listAllLoading; };
var getListAllActive = function (state) { return state.listAllActive; };
var getListAllDataMap = function (state) { return state.listAllDataMap; };
var getListAllFiltered = createSelector(getListAll, getListAllDataMap, function (listAll, dataMap) {
    return dataMap && listAll ? dataMap(listAll) : listAll;
});
var getSummary$1 = function (state) { return state.summary; };
var getSummaryActive$1 = function (state) { return state.summaryActive; };
var getSummaryQuery = function (state) { return state.summaryQuery; };
var getSummaryLoading$1 = function (state) { return state.summaryLoading; };
var getGraph = function (state) { return state.graph; };
var getGraphQuery = function (state) { return state.graphQuery; };
var getGraphLoading = function (state) { return state.graphLoading; };
var getGraphActive = function (state) { return state.graphActive; };

var reducers = {};
function reducer$1(state, action) {
    return combineReducers(reducers);
}
var metaReducers = [];
/**
 * Users selectors
 */
var getUsersState = createFeatureSelector("users");
var getUsersUsersData = createSelector(getUsersState, getUserData);
var getUsersPlacelineId = createSelector(getUsersState, getPlacelineId);
var getUsersPlacelineQuery = createSelector(getUsersState, getPlacelineQuery);
var getUsersPlacelineLoading = createSelector(getUsersState, getPlacelineLoading);
var getUsersListActive = createSelector(getUsersState, getListActive);
var getUsersListId = createSelector(getUsersState, getListId);
var getUsersListQuery = createSelector(getUsersState, getListQuery);
var getUsersAnalyticsPage = createSelector(getUsersState, getAnalyticsPage);
var getUsersAnalyticsLoading = createSelector(getUsersState, getAnalyticsLoading);
var getUsersIndexPage = createSelector(getUsersState, getIndexPage);
// export const getUsersListApiType = createSelector(getUsersState, fromUsers.getListApiType);
var getUsersIndexIsActive = createSelector(getUsersState, getIndexActive);
var getUsersAnalyticsIsActive = createSelector(getUsersState, getAnalyticsActive);
var getUsersIndexMarkersIsActive = createSelector(getUsersState, getIndexMarkersActive);
var getUsersAnalyticsMarkersIsActive = createSelector(getUsersState, getAnalyticsMarkersActive);
var getUsersAnalyticsAllLoading = createSelector(getUsersState, getAnalyticsAllLoading);
var getUsersIndexFilteredMarker = createSelector(getUsersState, getIndexFilteredMarkers);
var getUsersAnalyticsFilteredMarker = createSelector(getUsersState, getAnalyticFilteredMarkers);
var getUsersIndexAll = createSelector(getUsersState, getIndexAll);
var getUsersAnalyticsAll = createSelector(getUsersState, getAnalyticsAll);
var getUsersSummary = createSelector(getUsersState, getSummary);
var getUsersSummaryActive = createSelector(getUsersState, getSummaryActive);
var getUsersSummaryLoading = createSelector(getUsersState, getSummaryLoading);
/**
 * Segment selectors
 */
var getSegmentsState = createFeatureSelector("segments");
var getSegmentsSelectedId = createSelector(getSegmentsState, getSelectedId);
var getSegmentsResetMapId = createSelector(getSegmentsState, getResetMapId);
/**
 *
 * Groups reducer
 */
var getGroupsState = createFeatureSelector("groups");
var getGroupId = createSelector(getGroupsState, getId);
var getGroupAll = createSelector(getGroupsState, getAllGroups);
var getGroupListActive = createSelector(getGroupsState, getListActive$1);
/**
 Account
 */
var getAccountState = createFeatureSelector("accounts");
var getAccountUser$1 = createSelector(getAccountState, getAccountUser);
var getAccountMembershipsAll = createSelector(getAccountState, getMembershipsAll);
var getAccountKey = createSelector(getAccountState, getKey);
var getAccountTempKey = createSelector(getAccountState, getTempKey);
var getAccountCurrentKey = createSelector(getAccountState, getCurrentKey);
var getAccountUserId = createSelector(getAccountState, getUserId);
/**
 * Actions
 */
var getActionsState = createFeatureSelector("actions");
var getActionsList = createSelector(getActionsState, getList);
var getActionsListLoading = createSelector(getActionsState, getListLoading);
var getActionsListActive = createSelector(getActionsState, getListActive$2);
var getActionsListQuery = createSelector(getActionsState, getListQuery$1);
var getActionsListAll = createSelector(getActionsState, getListAll);
var getActionsListAllActive = createSelector(getActionsState, getListAllActive);
var getActionsListAllLoading = createSelector(getActionsState, getListAllLoading);
var getActionsListAllFiltered = createSelector(getActionsState, getListAllFiltered);
var getActionsSummary = createSelector(getActionsState, getSummary$1);
var getActionsSummaryActive = createSelector(getActionsState, getSummaryActive$1);
var getActionsSummaryLoading = createSelector(getActionsState, getSummaryLoading$1);
var getActionsSummaryQuery = createSelector(getActionsState, getSummaryQuery);
var getActionsGraph = createSelector(getActionsState, getGraph);
var getActionsGraphLoading = createSelector(getActionsState, getGraphLoading);
var getActionsGraphActive = createSelector(getActionsState, getGraphActive);
var getActionsGraphQuery = createSelector(getActionsState, getGraphQuery);

var EntityItemClient = /** @class */ (function () {
    function EntityItemClient() {
        //listGetData
        this.updateStrategy = "live";
        this.pollDuration = 10000;
        // api$;
        this.name = "item";
        this.allowedQueryKeys = null;
        // setData(data) {
        //
        // };
        // setActive(isActive: boolean = true){
        //
        // }
    }
    // init: () => void;
    // getData$: (data) => any;
    EntityItemClient.prototype.firstDataEffect = function () {
        this.setLoading(false);
    };
    EntityItemClient.prototype.getDefaultQuery = function () {
        return {};
    };
    Object.defineProperty(EntityItemClient.prototype, "apiParams$", {
        get: function () {
            return this.getApiParams$();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityItemClient.prototype, "apiQuery$", {
        get: function () {
            return this.getApiQuery$();
        },
        enumerable: true,
        configurable: true
    });
    return EntityItemClient;
}());

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$5 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
function getIdQueryDataMixin(Base) {
    return /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_1.prototype.getData$ = function (_a) {
            var _this = this;
            var id = _a[0], query = _a[1];
            var update = this.getFirstData$([id, query]).pipe(expand(function (data) {
                return timer(_this.pollDuration).pipe(switchMap(function () {
                    return _this.api$(id, query);
                }));
            }));
            return this.updateStrategy != "once"
                ? update
                : this.getFirstData$([id, query]);
        };
        return class_1;
    }(Base));
}
function getPageDataMixin(Base) {
    return /** @class */ (function (_super) {
        __extends(class_2, _super);
        function class_2() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_2.prototype.getData$ = function (_a) {
            var _this = this;
            var query = _a[0];
            var update = this.getFirstData$([query]).pipe(expand(function (data) {
                return timer(_this.pollDuration).pipe(switchMap(function () {
                    if (_this.updateStrategy == "live") {
                        return _this.api$(query);
                    }
                    else {
                        var ids = map(data.results, function (item) {
                            return item["id"];
                        });
                        var updateQuery = __assign$5({}, query, { id: ids.toString(), status: null, page: null });
                        return _this.api$(updateQuery).pipe(map$1(function (newData) {
                            return __assign$5({}, data, { results: newData.results });
                        }));
                    }
                }));
            }));
            return this.updateStrategy != "once"
                ? update
                : this.getFirstData$([query]);
        };
        return class_2;
    }(Base));
}
function getQueryDataMixin(Base) {
    return /** @class */ (function (_super) {
        __extends(class_3, _super);
        function class_3() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_3.prototype.getData$ = function (_a) {
            var _this = this;
            var query = _a[0];
            var update = this.getFirstData$([query]).pipe(expand(function (data) {
                return timer(_this.pollDuration).pipe(switchMap(function () {
                    return _this.api$(query);
                }));
            }));
            return this.updateStrategy != "once"
                ? update
                : this.getFirstData$([query]);
        };
        return class_3;
    }(Base));
}
function getAllPageDataMixin(Base) {
    return /** @class */ (function (_super) {
        __extends(class_4, _super);
        function class_4() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_4.prototype.getData$ = function (_a) {
            var query = _a[0];
            return this.getFirstData$([query]);
        };
        return class_4;
    }(Base));
}

var __assign$6 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var MergeQuery = function (defaultQuery) {
    return function (query$) {
        return query$.pipe(map$1(function (query) {
            return __assign$6({}, defaultQuery, query);
        }));
    };
};
// export const itemAsPage = <T>() => {
//   return (item$): Observable<Page<T>> => {
//     return item$.pipe(
//       map((item) => {
//         return item ?
//           {
//             count: 1,
//             next: null,
//             previous: null,
//             results: [item]
//           } : null
//       })
//     )
//   }
// };

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function listQueryMixin(Base) {
    return /** @class */ (function (_super) {
        __extends$1(class_1, _super);
        function class_1() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_1.prototype.getApiQuery$ = function () {
            return this.getApiParams$().pipe(map$1(function (data) {
                return data[0];
            }));
        };
        class_1.prototype.getBaseQuery$ = function () {
            var baseQuery$ = this.query$.pipe(AllowedQueryMap(this.allowedQueryMap), MergeQuery(this.getDefaultQuery()), CombineQueries([this.dateRange.data$.pipe(DateRangeToQuery$(this.dateParam)) || of({})]));
            return baseQuery$;
        };
        class_1.prototype.getApiParams$ = function () {
            var baseQuery$ = this.getBaseQuery$().pipe(map$1(function (data) {
                return [data];
            }));
            return this.active$
                ? this.active$.pipe(switchMap(function (isActive) {
                    // console.log(isActive, "accc");
                    return isActive ? baseQuery$ : of([]);
                }))
                : baseQuery$;
        };
        return class_1;
    }(Base));
}
function itemQueryMixin(Base) {
    return /** @class */ (function (_super) {
        __extends$1(class_2, _super);
        function class_2() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_2.prototype.getApiQuery$ = function () {
            return this.getApiParams$().pipe(map$1(function (data) {
                return data[1];
            }));
        };
        class_2.prototype.getApiParams$ = function () {
            return combineLatest(this.id$, this.query$.pipe(MergeQuery(this.getDefaultQuery())));
        };
        return class_2;
    }(Base));
}

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function clientSubMixin(Base) {
    return /** @class */ (function (_super) {
        __extends$2(class_1, _super);
        function class_1() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _this = _super.apply(this, args) || this;
            _this.init();
            return _this;
        }
        class_1.prototype.init = function () {
            var _this = this;
            if (!this.dataSub) {
                this.dataSub = this.getApiParams$()
                    .pipe(switchMap(function (data) {
                    var validApi = _this.validQueryData && data ? _this.validQueryData(data) : data && data[0];
                    if (validApi) {
                        var loading = typeof data[0] === "string" ? data[0] : true;
                        _this.setLoading(loading);
                        return _this.getData$(data);
                    }
                    else {
                        _this.setLoading(false);
                        return EMPTY;
                    }
                }))
                    .subscribe(function (data) {
                    _this.setData(data);
                });
            }
        };
        return class_1;
    }(Base));
}

var __extends$3 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function getFirstDataMixin(Base) {
    return /** @class */ (function (_super) {
        __extends$3(class_1, _super);
        function class_1() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        class_1.prototype.getFirstData$ = function (args) {
            var _this = this;
            var entity = this;
            return entity.api$.apply(entity, args).pipe(tap(function (data) {
                _this.firstDataEffect(data);
            }));
        };
        return class_1;
    }(Base));
}
//
// export interface IQueryFirstDataBase {
//   setLoading(any): void,
//   api$(query): Observable<any>
// }
//
// export function getQueryFirstDataMixin <TBase extends Constructor<IQueryFirstDataBase>>(Base: TBase) {
//   return class extends Base {
//     getFirstData$([query]) {
//       let entity = this;
//       return entity.api$(query).pipe(
//         tap((data) => {
//           this.setLoading(false);
//         })
//       );
//     }
//   }
// }

var __extends$4 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var UsersPlaceline = /** @class */ (function (_super) {
    __extends$4(UsersPlaceline, _super);
    function UsersPlaceline(_a) {
        var store = _a.store, api = _a.api;
        var _this = _super.call(this) || this;
        _this.name = "users placeline";
        _this.updateStrategy = "live";
        _this.api$ = function (id, query) { return api.placeline(id, query); };
        _this.store = store;
        _this.query$ = _this.store.select(getUsersPlacelineQuery);
        _this.actionId$ = _this.getQueryType('action_id');
        _this.data$ = _this.store.select(getUsersUsersData);
        _this.loading$ = _this.store.select(getUsersPlacelineLoading);
        _this.id$ = _this.store.select(getUsersPlacelineId);
        _this.segmentsState$ = _this.store.select(getSegmentsState);
        _this.segmentSelectedId$ = _this.store.select(getSegmentsSelectedId);
        _this.segmentResetId$ = _this.store.select(getSegmentsResetMapId);
        _this.hasData$ = combineLatest(_this.id$, _this.query$).pipe(map$1(function (_a) {
            var id = _a[0], query = _a[1];
            // console.log("here", id, query, this.validQueryData([id, query || {}]));
            return !!_this.validQueryData([id, query || {}]);
        }), distinctUntilChanged()
        // share
        );
        return _this;
        // this.init()
    }
    UsersPlaceline.prototype.getQueryType = function (type) {
        return this.query$.pipe(map$1(function (query) {
            return query ? query[type] : null;
        }), distinctUntilChanged());
    };
    UsersPlaceline.prototype.setData = function (data) {
        this.store.dispatch(new SetUserData(data));
    };
    UsersPlaceline.prototype.setLoading = function (data) {
        this.store.dispatch(new SetPlacelineLoading(data));
    };
    UsersPlaceline.prototype.setId = function (id) {
        this.store.dispatch(new SetPlacelineId(id));
        if (!id)
            this.setLoading(false);
    };
    UsersPlaceline.prototype.toggleId = function (userId) {
        this.store.dispatch(new TogglePlacelineId(userId));
    };
    UsersPlaceline.prototype.setQuery = function (query) {
        this.store.dispatch(new SetPlacelineQuery(query));
    };
    UsersPlaceline.prototype.setSegmentSelectedId = function (segmentId) {
        this.store.dispatch(new SetSelectedId(segmentId));
    };
    UsersPlaceline.prototype.setSegmentResetMapId = function (segmentId) {
        this.store.dispatch(new SetResetMapId(segmentId));
    };
    UsersPlaceline.prototype.getMapData$ = function () {
        // return dataWithSelectedId$(this.data$, this.segmentSelectedId$, [
        //   "segments"
        // ]);
    };
    UsersPlaceline.prototype.validQueryData = function (_a) {
        var id = _a[0], query = _a[1];
        return !!id || query.action_id || query.action_unique_id;
    };
    UsersPlaceline.prototype.clearData = function () {
        this.setData(null);
        this.setLoading(false);
        this.setQuery({});
        this.setId(null);
    };
    return UsersPlaceline;
}(EntityItemClient));
var UsersPlacelineClient = clientSubMixin(getIdQueryDataMixin(getFirstDataMixin(itemQueryMixin(UsersPlaceline))));

var EntityListClient = /** @class */ (function () {
    function EntityListClient() {
        //listGetData
        this.updateStrategy = "live";
        this.pollDuration = 10000;
        // api$;
        this.name = "list";
        //
        // setData(data) {
        //
        // };
        //
        // setActive(isActive: boolean = true){
        //
        // }
    }
    // init: () => void;
    // getData$: (data) => any;
    // dataSub: Subscription;
    EntityListClient.prototype.firstDataEffect = function (data) {
        this.setLoading(false);
    };
    EntityListClient.prototype.getDefaultQuery = function () {
        return { page_size: 10 };
    };
    Object.defineProperty(EntityListClient.prototype, "apiParams$", {
        get: function () {
            return this.getApiParams$();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityListClient.prototype, "apiQuery$", {
        get: function () {
            return this.getApiQuery$();
        },
        enumerable: true,
        configurable: true
    });
    return EntityListClient;
}());

var __extends$5 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$7 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var UsersAnalytics = /** @class */ (function (_super) {
    __extends$5(UsersAnalytics, _super);
    function UsersAnalytics(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        var _this = _super.call(this) || this;
        _this.name = "users analytics list";
        _this.defaultQuery = { ordering: "-last_heartbeat_at" };
        _this.api$ = function (query) {
            return api.analytics(query);
        };
        _this.dateRange = dateRange;
        _this.store = store;
        _this.query$ = _this.store.select(getUsersListQuery);
        _this.dateParam = dateParam;
        _this.active$ = _this.store.select(getUsersAnalyticsIsActive);
        _this.data$ = _this.store.select(getUsersAnalyticsPage);
        _this.id$ = _this.store.select(getUsersListId);
        _this.loading$ = _this.store.select(getUsersAnalyticsLoading);
        _this.dataArray$ = _this.data$.pipe(PageResults$);
        return _this;
        // this.init()
    }
    UsersAnalytics.prototype.getDefaultQuery = function () {
        return __assign$7({}, _super.prototype.getDefaultQuery.call(this), this.defaultQuery);
        // return {...this.defaultQuery}
    };
    UsersAnalytics.prototype.firstDataEffect = function () {
        this.setLoading(false);
    };
    UsersAnalytics.prototype.setLoading = function (data) {
        this.store.dispatch(new SetUsersAnalyticsLoading(data));
    };
    UsersAnalytics.prototype.setData = function (data) {
        this.store.dispatch(new SetUsersAnalyticsPage(data));
    };
    UsersAnalytics.prototype.setActive = function (isActive) {
        if (isActive === void 0) { isActive = true; }
        isActive = isActive ? new Date().toISOString() : isActive;
        this.store.dispatch(new SetListActive(isActive));
        if (!isActive)
            this.setLoading(false);
    };
    Object.defineProperty(UsersAnalytics.prototype, "apiQuery$", {
        get: function () {
            return this.getApiQuery$();
        },
        enumerable: true,
        configurable: true
    });
    UsersAnalytics.prototype.addQuery = function (query) {
        if (query === void 0) { query = {}; }
        this.store.dispatch(new AddListQuery(query));
    };
    UsersAnalytics.prototype.setQuery = function (query) {
        if (query === void 0) { query = {}; }
        this.store.dispatch(new SetListQuery(query));
    };
    UsersAnalytics.prototype.setQueryReset = function (query) {
        this.store.dispatch(new AddListQuery(__assign$7({}, query, { page: null })));
    };
    UsersAnalytics.prototype.clearQueryKey = function (key) {
        this.store.dispatch(new ClearQueryKey(key));
    };
    UsersAnalytics.prototype.toggleId = function (userId) {
        this.store.dispatch(new ToggleUsersListId(userId));
    };
    UsersAnalytics.prototype.setId = function (userId) {
        this.store.dispatch(new SetUsersListId(userId));
    };
    UsersAnalytics.prototype.clearData = function () {
        this.setActive(false);
        this.setData(null);
        // this.setLoading(false);
        // this.setQuery({});
    };
    UsersAnalytics.prototype.destroy = function () {
        this.clearData();
        this.dataSub.unsubscribe();
    };
    return UsersAnalytics;
}(EntityListClient));
var UsersAnalyticsClient = clientSubMixin(getPageDataMixin(getFirstDataMixin(listQueryMixin(UsersAnalytics))));
// applyMixins(UsersAnalytics, [ListGetData, ListQuery, ClientSub]);

var __assign$8 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var EntityClient = /** @class */ (function () {
    function EntityClient() {
    }
    EntityClient.prototype.dataArrayWithSelected$ = function (id$, dataArray$, selected$) {
        var userId$ = id$;
        var placelinePage$ = selected$.pipe(distinctUntilChanged(), map$1(function (data) {
            return data ? [data] : null;
        })); //todo take query from placeline
        var array$ = combineLatest(placelinePage$, userId$, dataArray$, function (placelinePage, userId, dataArray) {
            var filteredData = dataArray.filter(function (user) {
                return userId ? user.id == userId : true;
            });
            return placelinePage && userId ? placelinePage : filteredData;
        });
        return array$;
    };
    EntityClient.prototype.pageDataWithSelected$ = function (id$, pageData$, selected$) {
        // const userId$ = id$;
        var placelinePage$ = selected$.pipe(itemAsPage$());
        var newPageData$ = combineLatest(placelinePage$, id$, pageData$, function (placelinePage, userId, pageData) {
            if (!pageData)
                return pageData;
            var placelineResults = placelinePage ? placelinePage.results : null;
            var filteredData = pageData.results.filter(function (user) {
                return userId ? user.id == userId : true;
            });
            var results = placelineResults && userId ? placelineResults : filteredData;
            var count = userId ? 0 : pageData.count;
            return __assign$8({}, pageData, { results: results, count: count });
        });
        return newPageData$;
    };
    EntityClient.prototype.getPageFromEntity = function (item$) {
        return item$;
    };
    return EntityClient;
}());

var __extends$6 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$9 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var EntityAllItemsClient = /** @class */ (function (_super) {
    __extends$6(EntityAllItemsClient, _super);
    // data$: Observable<Page<any>>;
    function EntityAllItemsClient() {
        var _this = _super.call(this) || this;
        _this.updateStrategy = "once";
        _this.allowedQueryKeys = ["search", "status"];
        return _this;
    }
    Object.defineProperty(EntityAllItemsClient.prototype, "data$", {
        get: function () {
            return this.dataEntities$.pipe(map$1(function (allData) {
                if (!allData)
                    return allData;
                var results = values(allData.resultsEntity);
                return {
                    count: allData.count || 0,
                    results: results,
                    previous: allData.previous || "",
                    next: allData.next || ""
                };
            }));
        },
        enumerable: true,
        configurable: true
    });
    EntityAllItemsClient.prototype.getDefaultQuery = function () {
        return __assign$9({}, _super.prototype.getDefaultQuery.call(this), { page_size: 100, ordering: "-created_at" });
    };
    EntityAllItemsClient.prototype.firstDataEffect = function (data) {
        if ((data && !data.next) || !data) {
            this.setLoading(false);
        }
    };
    Object.defineProperty(EntityAllItemsClient.prototype, "dataArray$", {
        get: function () {
            var dataArray$ = this.data$.pipe(filter$1(function (data) { return !!data; }), map$1(function (data) {
                var resutls = values(data.resultsEntity);
                return resutls;
            }));
            return dataArray$;
        },
        enumerable: true,
        configurable: true
    });
    EntityAllItemsClient.prototype.getAllMarkers$ = function () {
        var _this = this;
        var allMarkers$ = this.dataArray$.pipe(map$1(function (markers) {
            return markers.reduce(function (acc, marker) {
                var isValid = _this.isValidMarker(marker);
                if (isValid) {
                    acc.valid.push(marker);
                }
                else {
                    acc.invalid.push(marker);
                }
                return acc;
            }, { valid: [], invalid: [] });
            // return markers
        }));
        return allMarkers$;
    };
    EntityAllItemsClient.prototype.isValidMarker = function (marker) {
        return true;
    };
    EntityAllItemsClient.prototype.getMarkers$ = function () {
        return this.getAllMarkers$().pipe(pluck("valid"));
    };
    return EntityAllItemsClient;
}(EntityListClient));

var __extends$7 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function listAllClientSubMixin(Base) {
    return /** @class */ (function (_super) {
        __extends$7(class_1, _super);
        function class_1() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _this = _super.apply(this, args) || this;
            _this.init();
            return _this;
        }
        class_1.prototype.init = function () {
            var _this = this;
            if (!this.dataSub) {
                this.dataSub = this.getApiParams$()
                    .pipe(switchMap(function (data) {
                    if (data && data[0]) {
                        var loading = typeof data[0] === "string" ? data[0] : true;
                        _this.setLoading(loading);
                        return _this.getData$(data);
                    }
                    else {
                        return EMPTY;
                    }
                }))
                    .subscribe(function (data) {
                    _this.addData(data);
                });
            }
        };
        return class_1;
    }(Base));
}

var __extends$8 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$10 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var UsersAnalyticsListAll = /** @class */ (function (_super) {
    __extends$8(UsersAnalyticsListAll, _super);
    function UsersAnalyticsListAll(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        var _this = _super.call(this) || this;
        _this.id$ = EMPTY;
        _this.allowedQueryKeys = ["status", "show_all", "search"];
        _this.name = "users analytics all";
        _this.defaultQuery = {};
        _this.allowedQueryMap = [
            {
                key: "status",
                filter: function (value, oldValue) { return !!value; }
            },
            {
                key: "show_all",
                filter: function (value, oldValue) {
                    if (!value)
                        _this.clearData();
                    return true;
                }
            }
        ];
        _this.api$ = function (query) { return api.allPages(api.analytics(query)); };
        _this.dateRange = dateRange;
        _this.store = store;
        _this.query$ = _this.store.select(getUsersListQuery);
        _this.dateParam = dateParam;
        _this.active$ = _this.store.select(getUsersAnalyticsMarkersIsActive);
        _this.dataEntities$ = _this.store.select(getUsersAnalyticsFilteredMarker);
        // this.id$ = this.store.select(fromRoot.getQueryUserId);
        _this.loading$ = _this.store.select(getUsersAnalyticsAllLoading);
        _this.dateRange.data$.subscribe(function (_) {
            _this.setData(null);
        });
        return _this;
        // this.dataArray$ = this.data$.pipe(PageResults$);
        // this.init()
    }
    UsersAnalyticsListAll.prototype.getDefaultQuery = function () {
        return __assign$10({}, _super.prototype.getDefaultQuery.call(this), this.defaultQuery);
        // return {...this.defaultQuery}
    };
    UsersAnalyticsListAll.prototype.firstDataEffect = function (data) {
        if (!data.next)
            this.setLoading(false);
    };
    UsersAnalyticsListAll.prototype.setActive = function (isActive) {
        if (isActive === void 0) { isActive = true; }
        isActive = isActive ? new Date().toISOString() : isActive;
        this.store.dispatch(new SetMarkersActive(isActive));
        if (!isActive)
            this.setLoading(false);
    };
    UsersAnalyticsListAll.prototype.addData = function (data) {
        data = data || { results: [], next: "no_next", count: 0, previous: "" };
        this.store.dispatch(new AddUsersAnalyticsAll(data));
    };
    UsersAnalyticsListAll.prototype.setData = function (data) {
        data = data || { results: [], next: "no_next", count: 0, previous: "" };
        this.store.dispatch(new SetUsersAnalyticsAll(data));
    };
    UsersAnalyticsListAll.prototype.setLoading = function (data) {
        this.store.dispatch(new SetUsersAnalyticsAllLoading(data));
    };
    UsersAnalyticsListAll.prototype.setDataMap = function (mapFunc) {
        this.store.dispatch(new SetUsersMarkersDataMap(mapFunc));
    };
    UsersAnalyticsListAll.prototype.isValidMarker = function (marker) {
        return htUser(marker).isValidMarker();
    };
    UsersAnalyticsListAll.prototype.clearData = function () {
        this.setActive(false);
        var nullData = { results: [], next: "no_next", count: 0, previous: "" };
        this.setLoading(false);
        this.setData(nullData);
    };
    UsersAnalyticsListAll.prototype.destroy = function () {
        this.clearData();
        this.setActive(false);
        this.dataSub.unsubscribe();
    };
    return UsersAnalyticsListAll;
}(EntityAllItemsClient));
var UsersAnalyticsListAllClient = listAllClientSubMixin(getAllPageDataMixin(getFirstDataMixin(listQueryMixin(UsersAnalyticsListAll))));

var __assign$11 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var BaseFilter = /** @class */ (function () {
    function BaseFilter() {
        this.sortingQueryMap = {};
    }
    Object.defineProperty(BaseFilter.prototype, "sortingQueryLabel", {
        get: function () {
            return this.getLabelArrayFromMap(this.sortingQueryMap);
        },
        enumerable: true,
        configurable: true
    });
    BaseFilter.prototype.getLabelArrayFromMap = function (queryMap) {
        var keys = Object.keys(queryMap);
        return map(keys, function (key) {
            var label = queryMap[key] || key;
            return {
                value: key,
                label: label
            };
        });
    };
    BaseFilter.prototype.getQueryDisplay = function (queryArray, key) {
        return map(queryArray, function (queryLabel) {
            var value = queryLabel.values ? queryLabel.values.toString() : queryLabel.value;
            return __assign$11({}, queryLabel, { param: (_a = {}, _a[key] = value, _a) });
            var _a;
        });
    };
    BaseFilter.prototype.getQueryLabel = function (query) {
        var _this = this;
        var keys = Object.keys(query);
        return keys.reduce(function (acc, key) {
            var value = query[key] || key;
            var label = _this.getQueryLabelFromValue(value, key);
            var queryLabel = {
                label: label,
                values: [key],
                value: key,
                param: (_a = {}, _a[key] = value, _a)
            };
            return label ? acc.concat([queryLabel]) : acc;
            var _a;
        }, []);
    };
    BaseFilter.prototype.getQueryLabelFromValue = function (value, key) {
        if (key === "search")
            return value;
        if (key === "show_all")
            return "Show All"; //todo add this after clear on show all removed
        var queryLabel = this.allQueryArray.find(function (queryLabel) {
            var valueString;
            if (queryLabel.values) {
                valueString = queryLabel.values.toString();
            }
            else if (queryLabel.values) {
                valueString = queryLabel.values;
            }
            else {
                return false;
            }
            return valueString === value;
        });
        return queryLabel ? queryLabel.label : null;
    };
    BaseFilter.prototype.summaryCharts = function (queryLabels, data, status) {
        if (data) {
            var max_1;
            var total_1 = 0;
            var values$$1 = queryLabels.map(function (entity) {
                var sum = entity.values.reduce(function (acc, key) {
                    return acc + data[key];
                }, 0);
                var value = entity.value || 0 + sum;
                max_1 = max_1 && value < max_1 ? max_1 : value;
                total_1 = total_1 + value;
                return __assign$11({}, entity, { value: value });
            });
            var totalUsers = total_1;
            var hasSelected_1 = false;
            var chart = map(values$$1, function (datum) {
                var selected = false;
                if (status && status == datum.values.toString()) {
                    selected = true;
                    hasSelected_1 = true;
                }
                var w = max_1 ? datum.value / max_1 : 0;
                return __assign$11({}, datum, { w: w, selected: selected });
            });
            return { totalUsers: totalUsers, chart: chart, hasSelected: hasSelected_1 };
        }
        else {
            return data;
        }
        // return data;
    };
    return BaseFilter;
}());

var __extends$9 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DefaultUsersFilter = /** @class */ (function (_super) {
    __extends$9(DefaultUsersFilter, _super);
    function DefaultUsersFilter() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.customQueryArray = [];
        _this.statusQueryArray = [
            {
                label: "Stopped",
                values: ["stopped"],
                color: Color.stop
            },
            {
                label: "Moving",
                values: ["on_trip"],
                color: Color.blue
            },
            {
                label: "Logged off",
                values: ["logged_off"],
                color: "#8a91a0"
            },
            {
                label: "Location disabled",
                values: ["location_disabled"],
                color: Color.red
            },
            {
                label: "Network offline",
                values: ["network_offline"],
                color: "#ccc"
            }
        ];
        _this.activityQueryArray = [
            {
                label: "Logged in",
                values: ["stopped", "on_trip", "network_offline"],
                color: Color.blue
            },
            {
                label: "Logged off",
                values: ["logged_off"],
                color: "#a8a8a8"
            },
            {
                label: "Location disabled",
                values: ["location_disabled"],
                color: Color.red
            }
        ];
        _this.genericQueryArray = [
            {
                label: "Show all",
                values: ["show_all"]
            }
        ];
        _this.showAllQueryArray = [
            {
                label: "Never tracked",
                values: ["never_tracked"],
                color: "#575757"
            }
        ];
        _this.sortingQueryMap = {
            num_trips: "Trips",
            name: "Name",
            status: "Status",
            num_places: "Places",
            total_distance: "Distance",
            total_duration: "Duration",
            stop_duration: "Stop duration",
            location_disabled_duration: "Location disabled",
            network_offline_duration: "Network offline",
            num_actions: "Actions",
            last_heartbeat_at: "Last updated"
        };
        _this.mapQueries = [];
        _this.statusOverviewQueries = ["search", "show_all"];
        return _this;
    }
    Object.defineProperty(DefaultUsersFilter.prototype, "allQueryArray", {
        get: function () {
            return this.statusQueryArray.concat(this.genericQueryArray, this.showAllQueryArray, this.customQueryArray);
        },
        enumerable: true,
        configurable: true
    });
    DefaultUsersFilter.prototype.getStatusQueryArray = function (showAll) {
        if (showAll === void 0) { showAll = false; }
        return showAll
            ? this.statusQueryArray.concat(this.showAllQueryArray) : this.statusQueryArray;
    };
    return DefaultUsersFilter;
}(BaseFilter));

var __extends$10 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var UsersSummary = /** @class */ (function (_super) {
    __extends$10(UsersSummary, _super);
    function UsersSummary(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        var _this = _super.call(this) || this;
        _this.name = "users summary";
        _this.defaultQuery = { page_size: null };
        _this.updateStrategy = "live";
        _this.allowedQueryMap = [
            {
                key: "show_all",
            },
            {
                key: "search"
            }
        ];
        _this.filter = new DefaultUsersFilter();
        _this.showAll = false;
        _this.api$ = function (query) { return api.summary(query); };
        _this.dateRange = dateRange;
        _this.store = store;
        _this.dateParam = dateParam;
        _this.query$ = _this.store.select(getUsersListQuery);
        _this.active$ = _this.store.select(getUsersSummaryActive);
        _this.data$ = _this.store.select(getUsersSummary);
        _this.loading$ = _this.store.select(getUsersSummaryLoading);
        return _this;
    }
    UsersSummary.prototype.setActive = function (isActive) {
        if (isActive === void 0) { isActive = true; }
        isActive = isActive ? new Date().toISOString() : isActive;
        this.store.dispatch(new SetSummaryActive(isActive));
        if (!isActive)
            this.setLoading(false);
    };
    UsersSummary.prototype.setData = function (usersSummary) {
        this.store.dispatch(new SetUsersSummary(usersSummary));
    };
    UsersSummary.prototype.setLoading = function (data) {
        this.store.dispatch(new SetSummaryLoading(data));
    };
    UsersSummary.prototype.setQuery = function (query) { };
    UsersSummary.prototype.clearData = function () {
        // this.setLoading(false);
        this.setActive(false);
        this.setData(null);
    };
    UsersSummary.prototype.destroy = function () {
        this.clearData();
        this.setActive(false);
        this.dataSub.unsubscribe();
    };
    UsersSummary.prototype.getSummaryChart$ = function (queryLabels) {
        var _this = this;
        return this.listStatusOverview$().pipe(map$1(function (summaryData) {
            var finalQueryLabels = queryLabels || _this.filter.getStatusQueryArray(_this.showAll);
            return _this.filter.summaryCharts(finalQueryLabels, summaryData);
        }));
    };
    UsersSummary.prototype.listStatusOverview$ = function () {
        return this.data$.pipe(filter$1(function (data) { return !!data; }), map$1(function (summary) {
            return summary.status_overview;
        }));
    };
    return UsersSummary;
}(EntityListClient));
var UsersSummaryClient = clientSubMixin(getQueryDataMixin(getFirstDataMixin(listQueryMixin(UsersSummary))));

var __extends$11 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Dispatcher = /** @class */ (function (_super) {
    __extends$11(Dispatcher, _super);
    function Dispatcher() {
        return _super.call(this, { type: "INIT" }) || this;
    }
    Dispatcher.prototype.next = function (action) {
        if (typeof action === "undefined") {
            throw new TypeError("Actions must be objects");
        }
        else if (typeof action.type === "undefined") {
            throw new TypeError("Actions must have a type property");
        }
        _super.prototype.next.call(this, action);
    };
    return Dispatcher;
}(BehaviorSubject));

var __extends$12 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$12 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var ReducerObservable = /** @class */ (function (_super) {
    __extends$12(ReducerObservable, _super);
    function ReducerObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ReducerObservable;
}(Observable));
var ReducerManagerDispatcher = /** @class */ (function (_super) {
    __extends$12(ReducerManagerDispatcher, _super);
    function ReducerManagerDispatcher() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ReducerManagerDispatcher;
}(Dispatcher));
var UPDATE = "@ngrx/store/update-reducers";
var ReducerManager = /** @class */ (function (_super) {
    __extends$12(ReducerManager, _super);
    function ReducerManager(dispatcher, initialState, reducers, reducerFactory) {
        var _this = _super.call(this, reducerFactory(reducers, initialState)) || this;
        _this.dispatcher = dispatcher;
        _this.initialState = initialState;
        _this.reducers = reducers;
        _this.reducerFactory = reducerFactory;
        return _this;
    }
    ReducerManager.prototype.addFeature = function (_a) {
        var reducers = _a.reducers, reducerFactory = _a.reducerFactory, metaReducers = _a.metaReducers, initialState = _a.initialState, key = _a.key;
        var reducer = typeof reducers === "function"
            ? function (state, action) { return reducers(state || initialState, action); }
            : createReducerFactory(reducerFactory, metaReducers)(reducers, initialState);
        this.addReducer(key, reducer);
    };
    ReducerManager.prototype.removeFeature = function (_a) {
        var key = _a.key;
        this.removeReducer(key);
    };
    ReducerManager.prototype.addReducer = function (key, reducer) {
        this.reducers = __assign$12({}, this.reducers, (_a = {}, _a[key] = reducer, _a));
        this.updateReducers();
        var _a;
    };
    ReducerManager.prototype.removeReducer = function (key) {
        this.reducers = omit(this.reducers, key);
        this.updateReducers();
    };
    ReducerManager.prototype.updateReducers = function () {
        this.next(this.reducerFactory(this.reducers, this.initialState));
        this.dispatcher.next({ type: UPDATE });
    };
    ReducerManager.prototype.ngOnDestroy = function () {
        this.complete();
    };
    return ReducerManager;
}(BehaviorSubject));

var __extends$13 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ScannedActionsSubject = /** @class */ (function (_super) {
    __extends$13(ScannedActionsSubject, _super);
    function ScannedActionsSubject() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ScannedActionsSubject;
}(Subject));

var __extends$14 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var StateObservable = /** @class */ (function (_super) {
    __extends$14(StateObservable, _super);
    function StateObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return StateObservable;
}(Observable));
var State = /** @class */ (function (_super) {
    __extends$14(State, _super);
    function State(actions$, reducer$, scannedActions, initialState) {
        var _this = _super.call(this, initialState) || this;
        var actionsOnQueue$ = actions$.pipe(observeOn(queueScheduler));
        var withLatestReducer$ = actionsOnQueue$.pipe(withLatestFrom(reducer$));
        var seed = { state: initialState };
        var stateAndAction$ = withLatestReducer$.pipe(scan(reduceState, seed));
        _this.stateSubscription = stateAndAction$.subscribe(function (_a) {
            var state = _a.state, action = _a.action;
            _this.next(state);
            scannedActions.next(action);
        });
        return _this;
    }
    State.prototype.ngOnDestroy = function () {
        this.stateSubscription.unsubscribe();
        this.complete();
    };
    State.INIT = "INIT";
    return State;
}(BehaviorSubject));
function reduceState(stateActionPair, _a) {
    if (stateActionPair === void 0) { stateActionPair = { state: undefined }; }
    var action = _a[0], reducer = _a[1];
    var state = stateActionPair.state;
    return { state: reducer(state, action), action: action };
}

var __extends$15 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Store = /** @class */ (function (_super) {
    __extends$15(Store, _super);
    function Store(state$, actionsObserver, reducerManager) {
        var _this = _super.call(this) || this;
        _this.actionsObserver = actionsObserver;
        _this.reducerManager = reducerManager;
        _this.source = state$;
        return _this;
    }
    Store.prototype.select = function (pathOrMapFn) {
        var paths = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            paths[_i - 1] = arguments[_i];
        }
        return select.call.apply(select, [null, pathOrMapFn].concat(paths))(this);
    };
    Store.prototype.lift = function (operator) {
        var store = new Store(this, this.actionsObserver, this.reducerManager);
        store.operator = operator;
        return store;
    };
    Store.prototype.dispatch = function (action) {
        this.actionsObserver.next(action);
    };
    Store.prototype.next = function (action) {
        this.actionsObserver.next(action);
    };
    Store.prototype.error = function (err) {
        this.actionsObserver.error(err);
    };
    Store.prototype.complete = function () {
        this.actionsObserver.complete();
    };
    Store.prototype.addReducer = function (key, reducer) {
        this.reducerManager.addReducer(key, reducer);
    };
    Store.prototype.removeReducer = function (key) {
        this.reducerManager.removeReducer(key);
    };
    return Store;
}(Observable));
function select(pathOrMapFn) {
    var paths = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        paths[_i - 1] = arguments[_i];
    }
    return function selectOperator(source$) {
        var mapped$;
        if (typeof pathOrMapFn === 'string') {
            mapped$ = source$.pipe(pluck.apply(void 0, [pathOrMapFn].concat(paths)));
        }
        else if (typeof pathOrMapFn === 'function') {
            mapped$ = source$.pipe(map$1(pathOrMapFn));
        }
        else {
            throw new TypeError("Unexpected type '" + typeof pathOrMapFn + "' in select operator," +
                " expected 'string' or 'function'");
        }
        return mapped$.pipe(distinctUntilChanged());
    };
}

var Effects = /** @class */ (function () {
    function Effects(dispatcher) {
        // dispatcher.pipe(
        //   flatMap((action) => GetEffect(action)),
        //   filter(data => !!data)
        // ).subscribe((action) => {
        //   dispatcher.next(action)
        // })
    }
    return Effects;
}());

var storeFactory = function (reducers, config) {
    if (config === void 0) { config = {}; }
    var INITIAL_STATE = config.initialState;
    var metaReducers = config.metaReducers;
    var _REDUCER_FACTORY = config.reducerFactory
        ? config.reducerFactory
        : combineReducers;
    var REDUCER_FACTORY = createReducerFactory(_REDUCER_FACTORY, metaReducers);
    var dispatcher = new Dispatcher(); //actionSubject
    var EFFECTS_PROVIDERS = new Effects(dispatcher);
    var reducerManager = new ReducerManager(dispatcher, INITIAL_STATE, reducers, REDUCER_FACTORY);
    var SCANNED_ACTIONS_SUBJECT_PROVIDERS = new ScannedActionsSubject();
    var STATE_PROVIDERS = new State(dispatcher, reducerManager, SCANNED_ACTIONS_SUBJECT_PROVIDERS, INITIAL_STATE);
    var STORE_PROVIDERS = new Store(STATE_PROVIDERS, dispatcher, reducerManager);
    return {
        INITIAL_STATE: INITIAL_STATE,
        metaReducers: metaReducers,
        _REDUCER_FACTORY: _REDUCER_FACTORY,
        dispatcher: dispatcher,
        SCANNED_ACTIONS_SUBJECT_PROVIDERS: SCANNED_ACTIONS_SUBJECT_PROVIDERS,
        REDUCER_FACTORY: REDUCER_FACTORY,
        EFFECTS_PROVIDERS: EFFECTS_PROVIDERS,
        reducerManager: reducerManager,
        STATE_PROVIDERS: STATE_PROVIDERS,
        STORE_PROVIDERS: STORE_PROVIDERS
    };
};

// const storeProvider: StoreProvider = storeFactory(fromRoot.reducers);
// Object.freeze(storeProvider);
// export const store = storeProvider.STORE_PROVIDERS;
var apiStoreFactory = function () {
    var storeProvider = storeFactory(reducers);
    return storeProvider.STORE_PROVIDERS;
};
var ApiStoreService = (function () {
    var instance;
    return {
        getInstance: function () {
            if (!instance) {
                instance = apiStoreFactory();
            }
            return instance;
        },
        getNewInstance: function () {
            return apiStoreFactory();
        }
    };
})();

var __assign$13 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var defaultDateRange = __assign$13({}, DateRangeMap.today);
var DateRange = /** @class */ (function () {
    function DateRange(initialDate) {
        if (initialDate === void 0) { initialDate = {}; }
        this.dataBehaviour$ = new BehaviorSubject(__assign$13({}, defaultDateRange, initialDate));
        this.data$ = this.dataBehaviour$.asObservable();
    }
    DateRange.prototype.setDateRange = function (dateRange) {
        if (dateRange === void 0) { dateRange = null; }
        this.dataBehaviour$.next(dateRange);
    };
    DateRange.prototype.getDateRange = function () {
        return this.dataBehaviour$.getValue();
    };
    Object.defineProperty(DateRange.prototype, "display$", {
        get: function () {
            return this.data$.pipe(map$1(function (range) {
                return dateRangeDisplay(range);
            }));
        },
        enumerable: true,
        configurable: true
    });
    return DateRange;
}());
var dateRangeFactory = function (initialDate) {
    if (initialDate === void 0) { initialDate = {}; }
    return new DateRange(initialDate);
};
var dateRangeService = (function () {
    var instance;
    return {
        getInstance: function (initialDate) {
            if (!instance) {
                instance = dateRangeFactory(initialDate);
            }
            return instance;
        }
    };
})();

var UsersHeatmap = /** @class */ (function () {
    function UsersHeatmap(_a) {
        var dateRange = _a.dateRange, dateParam = _a.dateParam, api = _a.api;
        this.query$ = of({});
        this.loadingState$ = new BehaviorSubject(false);
        this.loading$ = this.loadingState$.asObservable();
        this.dataState$ = new BehaviorSubject(null);
        this.data$ = this.dataState$.asObservable();
        this.active$ = new BehaviorSubject(false);
        this.dataArray$ = this.data$.pipe(PageResults$);
        this.api$ = function (query) { return api.allPages(api.heatmap(query)); };
        this.dateRange = dateRange;
        this.dateParam = dateParam;
    }
    UsersHeatmap.prototype.setActive = function (active) {
        if (active === void 0) { active = true; }
        this.active$.next(active);
        if (!active)
            this.setLoading(false);
    };
    UsersHeatmap.prototype.getDefaultQuery = function () {
        return { page_size: 300 };
    };
    UsersHeatmap.prototype.firstDataEffect = function (data) {
        if ((data && !data.next) || !data) {
            this.setLoading(false);
        }
    };
    UsersHeatmap.prototype.setLoading = function (isLoading) {
        this.loadingState$.next(isLoading);
    };
    UsersHeatmap.prototype.setData = function (data) {
        this.dataState$.next(data);
    };
    UsersHeatmap.prototype.addData = function (data) {
        this.dataState$.next(data);
    };
    UsersHeatmap.prototype.clearData = function () {
        // this.setLoading(false);
        this.setActive(false);
        this.setData(null);
    };
    return UsersHeatmap;
}());
var UsersHeatmapClient = listAllClientSubMixin(getAllPageDataMixin(getFirstDataMixin(listQueryMixin(UsersHeatmap))));

var HtClient = /** @class */ (function () {
    function HtClient(_a) {
        var _b = _a === void 0 ? {} : _a, request = _b.request, token = _b.token;
        request = request || new HtRequest(token);
        this.api = new HtApi(request, token);
        this.token = token;
    }
    Object.defineProperty(HtClient.prototype, "token", {
        set: function (token) {
            this.api.request.tokenServie.token = token;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HtClient.prototype, "tempToken", {
        set: function (token) {
            this.api.request.tokenServie.tempToken = token;
        },
        enumerable: true,
        configurable: true
    });
    return HtClient;
}());
var initClient = function (options) {
    if (options === void 0) { options = {}; }
    return htClientService.setInstance(options);
};
var htClientFactory = function (options) {
    if (options === void 0) { options = {}; }
    return new HtClient(options);
};
var htClientService = (function () {
    var instance;
    return {
        setInstance: function (options) {
            if (options === void 0) { options = {}; }
            if (instance) {
                console.error("Client already initialized");
            }
            instance = htClientFactory(options);
            return instance;
        },
        getInstance: function () {
            if (!instance) {
                console.error("Client not initialized, call initClient(request: HtRequest, token?: string)");
            }
            return instance;
        }
    };
})();

var __extends$16 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$14 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var HtUsersClient = /** @class */ (function (_super) {
    __extends$16(HtUsersClient, _super);
    function HtUsersClient(options) {
        var _this = _super.call(this) || this;
        _this.options = options;
        _this.filterClass = new DefaultUsersFilter();
        _this.showAll = false;
        var api = htClientService.getInstance().api.users;
        _this.key$ = ApiStoreService.getInstance().select(getAccountCurrentKey);
        _this.api = api;
        var store = ApiStoreService.getNewInstance();
        store.addReducer("users", usersReducer);
        store.addReducer("segments", segmentsReducer);
        _this.store = store;
        var dateRange = _this.options.dateRange;
        _this.dateRange = dateRange;
        var dateParam = 'recorded_at';
        _this.analytics = new UsersAnalyticsClient({ dateRange: dateRange, store: store, dateParam: dateParam, api: api });
        _this.placeline = new UsersPlacelineClient({ store: store, api: api });
        _this.analyticsAll = new UsersAnalyticsListAllClient({
            dateRange: dateRange,
            store: store,
            dateParam: dateParam,
            api: api
        });
        _this.summary = new UsersSummaryClient({ dateRange: dateRange, store: store, dateParam: dateParam, api: api });
        _this.list = _this.analytics;
        _this.listAll = _this.analyticsAll;
        _this.heatmap = new UsersHeatmapClient({ dateRange: dateRange, dateParam: dateParam, api: api });
        _this.initEffects();
        return _this;
    }
    HtUsersClient.prototype.getLoading$ = function () {
        return CombineLoadings$(this.list.loading$, this.summary.loading$, this.placeline.loading$);
    };
    Object.defineProperty(HtUsersClient.prototype, "statusQueryArray", {
        set: function (data) {
            this._statusQueryArray = data;
            this.filterClass.customQueryArray = data;
        },
        enumerable: true,
        configurable: true
    });
    HtUsersClient.prototype.setShowAll = function (showAll) {
        if (showAll === void 0) { showAll = true; }
        this.list.setQuery({ show_all: true });
    };
    HtUsersClient.prototype.getInitialDateRange = function (range) {
        if (range === void 0) { range = {}; }
        var initialRange = DateRangeMap.today;
        return __assign$14({}, range, initialRange);
    };
    HtUsersClient.prototype.listStatusOverview$ = function () {
        return this.summary.data$.pipe(map$1(function (summary) {
            if (summary) {
                return summary.status_overview;
            }
            return null;
        }));
    };
    HtUsersClient.prototype.listStatusChart$ = function (queryLabels) {
        var _this = this;
        // return status_overview.
        if (queryLabels)
            (_a = this.filterClass.customQueryArray).push.apply(_a, queryLabels);
        return combineLatest(this.list.query$, this.listStatusOverview$()).pipe(map$1(function (_a) {
            var query = _a[0], overview = _a[1];
            if (overview) {
                var total_1 = 0;
                var max_1 = 0;
                var summaryEntity = queryLabels || _this.filterClass.getStatusQueryArray(_this.showAll);
                var status_1 = query ? query["status"] : null;
                // let summaryEntity = this.filterClass.activityQueryArray;
                var values$$1 = map(summaryEntity, function (entity) {
                    var sum = entity.values.reduce(function (acc, key) {
                        return acc + overview[key];
                    }, 0);
                    var value = entity.value || 0 + sum;
                    max_1 = max_1 && value < max_1 ? max_1 : value;
                    total_1 = total_1 + value;
                    return __assign$14({}, entity, { value: value });
                });
                var totalUsers = total_1;
                var hasSelected_1 = false;
                var chart = map(values$$1, function (datum) {
                    var selected = false;
                    if (status_1 && status_1 == datum.values.toString()) {
                        selected = true;
                        hasSelected_1 = true;
                    }
                    var w = max_1 ? datum.value / max_1 : 0;
                    return __assign$14({}, datum, { w: w, selected: selected });
                });
                return { totalUsers: totalUsers, chart: chart, hasSelected: hasSelected_1 };
            }
            return null;
        }));
        var _a;
    };
    Object.defineProperty(HtUsersClient.prototype, "queryLabel$", {
        get: function () {
            var _this = this;
            var query$ = this.list.getApiQuery$().pipe(filter$1(function (data) { return !!data; }));
            return query$.pipe(map$1(function (query) {
                // console.log("cl", query);
                var queryLabel = _this.filterClass.getQueryLabel(query);
                return queryLabel;
            }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HtUsersClient.prototype, "ordering$", {
        get: function () {
            var _this = this;
            return this.list.getApiQuery$().pipe(filter$1(function (data) { return !!data; }), map$1(function (query) {
                var ordering = query ? query["ordering"] : null;
                var orderingMod = _this.getOrderingMod(ordering);
                return {
                    string: _this.filterClass.sortingQueryMap[orderingMod.string],
                    sign: orderingMod.sign
                };
            }), distinctUntilChanged());
        },
        enumerable: true,
        configurable: true
    });
    HtUsersClient.prototype.getOrderingMod = function (ordering) {
        var string = ordering;
        var sign = 1;
        if (ordering.includes("-")) {
            string = ordering.substring(1);
            sign = 0;
        }
        return {
            string: string,
            sign: sign
        };
    };
    HtUsersClient.prototype.getSegmentsStates = function () {
        return this.store.select(getSegmentsState);
    };
    HtUsersClient.prototype.getCurrentPlaceline$ = function () {
        return combineLatest(this.placeline.data$, this.getSegmentsStates(), function (userData, _a) {
            var selectedId = _a.selectedId, resetMapId = _a.resetMapId;
            if (userData && (selectedId || resetMapId)) {
                var id_1 = selectedId || resetMapId;
                var placeline = userData.placeline.filter(function (segment) {
                    return segment.id === id_1;
                });
                userData = __assign$14({}, userData, { placeline: placeline, events: [], actions: [] });
            }
            return userData;
        });
    };
    HtUsersClient.prototype.initEffects = function () {
        var _this = this;
        /**
         * set show all if no results
         */
        this.list.data$.pipe(filter$1(function (data) { return !!data && data.results.length == 0; }), flatMap(function (noresults) {
            return _this.list.getApiQuery$().pipe(take(1), filter$1(function (query) {
                return ['status', 'search', 'show_all'].reduce(function (pass, param) {
                    return pass && !query[param];
                }, true);
            }));
        })).subscribe(function (_) {
            // console.log("set show all");
            _this.list.setQuery({ show_all: true });
        });
        this.list.query$.pipe(filter$1(function (data) { return !!data; })).subscribe(function (query) {
            _this.setListAllFilter(query);
        });
        this.list.query$.pipe(map$1(function (query) {
            return query ? query['show_all'] : query;
        }), distinctUntilChanged()).subscribe(function (showAll) {
            _this.showAll = showAll;
        });
        // this.listAll.active$.pipe(filter(data => !!data)).flatMap(() => {
        //   return this.listStatusChart$()
        // })
        //   .takeUntil(this.listAll.active$.filter(data => !data).skip(1))
        //   .withLatestFrom(this.list.query$)
        //   .switchMap(([statusOverview, query]) => {
        //     // return Observable.of({})
        //     console.log(statusOverview, query);
        //     return this.getListAllUpdateQuery$(statusOverview, query)
        //   })//todo finish this
        // todo include placeline query with id for action_id, action_unique_id case
        this.placeline.id$
            .pipe(scan(function (acc, currentId) {
            var isSame = acc.oldId === currentId;
            var oldId = currentId;
            return { isSame: isSame, oldId: oldId };
        }, { isSame: false, oldId: null }), pluck("isSame"), filter$1(function (data) { return !data; }))
            .subscribe(function (isDiff) {
            _this.placeline.setData(null);
        });
    };
    HtUsersClient.prototype.setListAllFilter = function (query) {
        var statusString = query["status"];
        var search = query["search"];
        var ids = query["id"];
        var userMarkerFilters = [];
        if (statusString) {
            var statusArray = statusString.split(",");
            // this.updateUserMap(query);
            var statusFilter_1 = [];
            statusArray.forEach(function (status) {
                statusFilter_1.push(htUser().getMarkerFilter(status));
            });
            var allStatusFilter = function (user) {
                return statusFilter_1.reduce(function (acc, filter$$1) {
                    return acc || filter$$1(user);
                }, false);
            };
            userMarkerFilters.push(allStatusFilter);
        }
        if (search) {
            userMarkerFilters.push(function (user) {
                return htUser().getMarkerSearched(search)(user); // || userMarkerFilter(user)
            });
            // this.updateUserMap(query);
        }
        if (ids) {
            var userIds_1 = ids.split(",");
            userMarkerFilters.push(function (user) {
                return userIds_1.includes(user.id);
            });
        }
        var userMarkerFilter = function (user) {
            return userMarkerFilters.reduce(function (acc, filter$$1) {
                return acc && filter$$1(user);
            }, true);
        };
        var dataMap = function (allResults) {
            var results = filter(allResults.resultsEntity, userMarkerFilter);
            var resultsEntity = indexBy(results, "id");
            return __assign$14({}, allResults, { resultsEntity: resultsEntity });
        };
        this.listAll.setDataMap(dataMap);
    };
    HtUsersClient.prototype.getListAllUpdateQuery$ = function (overview, query) {
        return this.listAll.data$.pipe(flatMap(function (allData) {
            var results = values(allData.resultsEntity);
            var currentTotalUsers = results.length;
            var totalUsers = overview.totalUsers, chart = overview.chart;
            var status = query["status"];
            if (!!status) {
                var value = chart.find(function (datum) {
                    return datum.keys.toString(",") == status;
                });
                return value && value !== currentTotalUsers ? of(true) : EMPTY;
            }
            else if (currentTotalUsers < totalUsers) {
                return of(true);
            }
            return EMPTY;
        }));
    };
    return HtUsersClient;
}(EntityClient));
var usersClientFactory = function (options) {
    if (options === void 0) { options = {}; }
    var dateRange = options.dateRange || dateRangeService.getInstance();
    return new HtUsersClient({ dateRange: dateRange });
};

var __extends$17 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ActionsFilter = /** @class */ (function (_super) {
    __extends$17(ActionsFilter, _super);
    function ActionsFilter() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.customQueryArray = [];
        _this.statusQueryArray = [
            // {
            //   label: "Not yet started",
            //   values: ["created"],
            //   color: "#8a91a0"
            // },
            {
                label: "Created",
                values: ["created"],
                color: Color.blue
            },
            {
                label: "Completed",
                values: ["completed", "autocompleted"],
                color: Color.stop
            },
            // {
            //   label: "AutoCompleted",
            //   values: ["autocompleted"],
            //   color: 'grey'
            //
            // },
            {
                label: "Canceled",
                values: ["canceled", "autocanceled"],
                color: Color.red
            },
        ];
        _this.sortingQueryMap = {
            created_at: "Created",
            completed_at: "Completed",
            distance: "Distance",
            duration: "Duration",
            type: "Type",
            user__name: "User name",
            status: "Status"
        };
        _this.genericQueryArray = [];
        return _this;
    }
    Object.defineProperty(ActionsFilter.prototype, "allQueryArray", {
        get: function () {
            return this.statusQueryArray.concat(this.genericQueryArray, this.customQueryArray);
        },
        enumerable: true,
        configurable: true
    });
    return ActionsFilter;
}(BaseFilter));

var __assign$15 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var GroupsList = /** @class */ (function () {
    function GroupsList(_a) {
        var store = _a.store, api = _a.api;
        this.name = "group";
        this.defaultQuery = { ordering: "-created_at" };
        this.query$ = of({});
        this.updateStrategy = "once";
        this.pollDuration = 10000;
        this.loading$ = of(false);
        this.api$ = function (query) { return api.index(query); };
        this.store = store;
        this.data$ = this.store.select(getGroupAll);
        this.active$ = this.store.select(getGroupListActive);
        this.dataArray$ = this.data$.pipe(PageResults$);
    }
    GroupsList.prototype.setData = function (data) {
        this.store.dispatch(new SetGroupsAll(data));
    };
    GroupsList.prototype.setLoading = function (data) {
        console.log("loading", data);
    };
    GroupsList.prototype.setActive = function (isActive) {
        if (isActive === void 0) { isActive = true; }
        this.store.dispatch(new SetListActive$1(isActive));
        if (!isActive)
            this.setLoading(false);
    };
    GroupsList.prototype.setQuery = function () { };
    GroupsList.prototype.firstDataEffect = function (data) {
        this.setLoading(false);
    };
    GroupsList.prototype.getRoots = function () {
        return this.api$({ has_parent: false });
    };
    GroupsList.prototype.getChildren = function (groupId) {
        return this.api$({ parent_group_id: groupId });
    };
    GroupsList.prototype.getDefaultQuery = function () {
        return __assign$15({ page_size: 10 }, this.defaultQuery);
    };
    GroupsList.prototype.clearData = function () {
        this.setActive(false);
        this.setData(null);
    };
    GroupsList.prototype.destroy = function () {
        this.clearData();
        this.setActive(false);
        this.dataSub.unsubscribe();
    };
    return GroupsList;
}());
var GroupsListClient = clientSubMixin(getPageDataMixin(getFirstDataMixin(listQueryMixin(GroupsList))));
// applyMixins(GroupsListClient, [ListGetData, ListQuery, ClientSub]);

var __extends$18 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$16 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var GroupsItem = /** @class */ (function (_super) {
    __extends$18(GroupsItem, _super);
    function GroupsItem(_a) {
        var store = _a.store, api = _a.api;
        var _this = _super.call(this) || this;
        _this.name = "group";
        _this.defaultQuery = { ordering: "-created_at" };
        _this.updateStrategy = "once";
        _this.query$ = of({});
        _this.data$ = EMPTY;
        _this.loading$ = of(false);
        _this.api$ = function (id, query) { return api.get(id, query); };
        _this.store = store;
        _this.id$ = _this.store.select(getGroupId);
        return _this;
        // this.init()
    }
    GroupsItem.prototype.getDefaultQuery = function () {
        return __assign$16({}, _super.prototype.getDefaultQuery.call(this), this.defaultQuery);
    };
    GroupsItem.prototype.setId = function (id) {
        this.store.dispatch(new SetGroupId(id));
    };
    GroupsItem.prototype.setData = function (data) {
        this.store.dispatch(new SetGroup(data));
    };
    GroupsItem.prototype.setLoading = function (data) { };
    GroupsItem.prototype.setQuery = function () { };
    GroupsItem.prototype.clearData = function () {
        this.setData(null);
    };
    GroupsItem.prototype.destroy = function () {
        this.clearData();
        this.dataSub.unsubscribe();
    };
    return GroupsItem;
}(EntityItemClient));
var GroupsItemClient = clientSubMixin(getIdQueryDataMixin(getFirstDataMixin(itemQueryMixin(GroupsItem))));
// applyMixins(GroupsItemClient, [ItemGetData, ItemQuery, ClientSub]);
// export const groupsItemsClientFactory = (config = {}): GroupsItem => {
//   let innerConfig = {
//     name: 'group',
//     defaultQuery: {ordering: '-created_at'},
//     updateStrategy: 'once',
//     ...config
//   };
//
//   let itemSelector: EntityItemSelectors = {
//     id$: store.select(fromGroup.getGroupId),
//     query$: Observable.of({}),
//     data$: Observable.empty(),
//     loading$: Observable.of(false),
//   };
//
//   let dispatchers: EntityItemDispatchers = {
//     setId(id) {
//       store.dispatch(new fromGroupDispatcher.SetGroupId(id))
//     },
//     setData(data) {
//       store.dispatch(new fromGroupDispatcher.SetGroup(data))
//     },
//     setLoading(data) {
//
//     },
//     setQuery() {
//
//     }
//
//   };
//
//   const state = {
//     api$: (id, query?) => clientApi.groups.get(id, query),
//     dispatchers,
//     selectors: itemSelector
//   };
//
//   let groupsIndex = entityClientFactory(state, innerConfig, 'item');
//   groupsIndex.init();
//   return groupsIndex as GroupsItem;
//
// };

var __extends$19 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var HtGroupsClient = /** @class */ (function (_super) {
    __extends$19(HtGroupsClient, _super);
    function HtGroupsClient(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        var api = htClientService.getInstance().api.groups;
        _this.api = api;
        var store = ApiStoreService.getNewInstance();
        store.addReducer("groups", groupsReducer);
        _this.store = store;
        _this.list = new GroupsListClient({ store: store, api: api });
        _this.item = new GroupsItemClient({ store: store, api: api });
        return _this;
    }
    HtGroupsClient.prototype.key$ = function (id) {
        return this.api.get(id).pipe(map$1(function (group) {
            return group["token"];
        }));
    };
    HtGroupsClient.prototype.lookupIdKey$ = function (lookupId) {
        return this.api.index({ lookup_id: lookupId }).pipe(map$1(function (groupPage) {
            return groupPage && groupPage["results"]
                ? groupPage["results"][0]["token"]
                : null;
        }));
    };
    HtGroupsClient.prototype.getChildren = function (groupId) {
        var query = { parent_group_id: groupId };
        return this.getAll(query);
    };
    HtGroupsClient.prototype.getRoot = function () {
        var query = { has_parent: false };
        return this.getAll(query);
    };
    HtGroupsClient.prototype.getAll = function (query) {
        return this.api.allPages(this.api.index(query));
    };
    return HtGroupsClient;
}(EntityClient));
var groupsClientFactory = function (options) {
    if (options === void 0) { options = {}; }
    var dateRange = options.noDateRange
        ? null
        : options.dateRange || dateRangeService.getInstance();
    return new HtGroupsClient({ dateRange: dateRange });
};

var __extends$20 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var AccountUser = /** @class */ (function (_super) {
    __extends$20(AccountUser, _super);
    function AccountUser(_a) {
        var dateParam = _a.dateParam, store = _a.store, api = _a.api;
        var _this = _super.call(this) || this;
        _this.query$ = of({});
        _this.updateStrategy = "once";
        _this.pollDuration = 10000;
        _this.api$ = function (id, query) { return api.get(id, query); };
        _this.store = store;
        // this.active$ = this.store.select(fromRoot.getUsersAnalyticsIsActive);
        _this.data$ = _this.store.select(getAccountUser$1);
        _this.id$ = _this.store.select(getAccountUserId);
        _this.loading$ = _this.store.select(getAccountCurrentKey);
        return _this;
        // this.init()
    }
    AccountUser.prototype.getDefaultQuery = function () {
        return {};
    };
    AccountUser.prototype.setId = function (id) {
        this.store.dispatch(new SetUserId(id));
    };
    AccountUser.prototype.setLoading = function (loading) { };
    AccountUser.prototype.setData = function (data) {
        this.store.dispatch(new SetAccountUser(data));
    };
    return AccountUser;
}(EntityItemClient));
var AccountUserClient = clientSubMixin(getIdQueryDataMixin(getFirstDataMixin(itemQueryMixin(AccountUser))));

var MembershipsAll = /** @class */ (function () {
    function MembershipsAll(_a) {
        var dateParam = _a.dateParam, store = _a.store, api = _a.api;
        this.query$ = of({});
        this.updateStrategy = "once";
        this.pollDuration = 10000;
        this.active$ = new BehaviorSubject(false);
        this.loading$ = new BehaviorSubject(false);
        this.api$ = function (id, query) { return api.membershipsAll(id, query); };
        this.store = store;
        // this.active$ = this.store.select(fromRoot.getUsersAnalyticsIsActive);
        this.data$ = this.store.select(getAccountMembershipsAll);
        this.id$ = this.store.select(getAccountUserId);
        // this.loading$ = this.store.select(fromRoot.getAccountCurrentKey);
        // this.init()
    }
    MembershipsAll.prototype.setActive = function (isActive) {
        this.active$.next(isActive);
        if (!isActive)
            this.setLoading(false);
    };
    MembershipsAll.prototype.firstDataEffect = function () {
        this.setLoading();
    };
    MembershipsAll.prototype.getDefaultQuery = function () {
        return { page_size: 100 };
    };
    MembershipsAll.prototype.setLoading = function (loading) {
        if (loading === void 0) { loading = true; }
        this.loading$.next(loading);
    };
    MembershipsAll.prototype.addData = function (data) {
        this.store.dispatch(new SetMembershipsAll(data));
    };
    return MembershipsAll;
}());
var MemberShipsClient = listAllClientSubMixin(getIdQueryDataMixin(getFirstDataMixin(itemQueryMixin(MembershipsAll))));

var AccountsClient = /** @class */ (function () {
    function AccountsClient() {
        var api = htClientService.getInstance().api.accountUser;
        this.api = api;
        this.store = ApiStoreService.getInstance();
        this.store.addReducer("accounts", reducer);
        this.accountUser = new AccountUserClient({ store: this.store, api: api });
        this.memberships = new MemberShipsClient({ store: this.store, api: api });
    }
    AccountsClient.prototype.setToken = function (token) {
        this.store.dispatch(new SetKey(token));
    };
    return AccountsClient;
}());

var ActionsGraph = /** @class */ (function () {
    function ActionsGraph(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        this.updateStrategy = "once";
        this.pollDuration = 10000;
        this.store = store;
        this.query$ = store.select(getActionsGraphQuery);
        this.active$ = this.store.select(getActionsGraphActive);
        this.loading$ = store.select(getActionsGraphLoading);
        this.data$ = store.select(getActionsGraph);
        this.api$ = function (query) { return api.graph(query); };
        this.dateRange = dateRange;
        this.dateParam = dateParam;
    }
    ActionsGraph.prototype.setActive = function (active) {
        if (active === void 0) { active = true; }
        this.store.dispatch(new SetGraphActive(active));
        if (!active)
            this.setLoading(false);
    };
    ActionsGraph.prototype.setQuery = function () {
    };
    ActionsGraph.prototype.getDefaultQuery = function () {
        return {};
    };
    ActionsGraph.prototype.setData = function (data) {
        this.store.dispatch(new SetGraph(data));
    };
    ActionsGraph.prototype.firstDataEffect = function () {
        this.setLoading(false);
    };
    ActionsGraph.prototype.setLoading = function (loading) {
        if (loading === void 0) { loading = true; }
        this.store.dispatch(new SetGraphLoading(!!loading));
    };
    ActionsGraph.prototype.clearData = function () {
        this.setData(null);
        this.setActive(false);
    };
    ActionsGraph.prototype.destroy = function () {
        this.clearData();
        this.dataSub.unsubscribe();
    };
    return ActionsGraph;
}());
var ActionsGraphClient = clientSubMixin(getQueryDataMixin(getFirstDataMixin(listQueryMixin(ActionsGraph))));

var __extends$21 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$17 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var ActionsList = /** @class */ (function (_super) {
    __extends$21(ActionsList, _super);
    function ActionsList(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        var _this = _super.call(this) || this;
        _this.defaultQuery = {
            ordering: '-created_at'
        };
        _this.api$ = function (query) {
            return api.index(query);
        };
        _this.dateRange = dateRange;
        _this.store = store;
        _this.query$ = _this.store.select(getActionsListQuery);
        _this.dateParam = dateParam;
        _this.active$ = _this.store.select(getActionsListActive);
        _this.data$ = _this.store.select(getActionsList);
        _this.loading$ = _this.store.select(getActionsListLoading);
        _this.dataArray$ = _this.data$.pipe(PageResults$);
        return _this;
        // this.init()
    }
    ActionsList.prototype.getDefaultQuery = function () {
        return __assign$17({}, _super.prototype.getDefaultQuery.call(this), this.defaultQuery);
        // return {...this.defaultQuery}
    };
    ActionsList.prototype.setActive = function (active) {
        if (active === void 0) { active = true; }
        this.store.dispatch(new SetListActive$2(active));
        if (!active)
            this.setLoading(false);
    };
    ActionsList.prototype.setLoading = function (loading) {
        this.store.dispatch(new SetListLoading(loading));
    };
    ActionsList.prototype.setData = function (data) {
        this.store.dispatch(new SetList(data));
    };
    ActionsList.prototype.addQuery = function (query) {
        this.store.dispatch(new AddListQuery$1(query));
    };
    ActionsList.prototype.setQuery = function (query) {
        this.store.dispatch(new SetListQuery$1(query));
    };
    ActionsList.prototype.clearQueryKey = function (key) {
        this.store.dispatch(new ClearQueryKey$1(key));
    };
    ActionsList.prototype.clearData = function () {
        this.setActive(false);
        this.setData(null);
    };
    return ActionsList;
}(EntityListClient));
var ActionsListClient = clientSubMixin(getPageDataMixin(getFirstDataMixin(listQueryMixin(ActionsList))));

var __extends$22 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ActionsSummary = /** @class */ (function (_super) {
    __extends$22(ActionsSummary, _super);
    function ActionsSummary(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        var _this = _super.call(this) || this;
        _this.allowedQueryMap = [
            {
                key: "show_all",
            },
            {
                key: "search"
            }
        ];
        _this.updateStrategy = 'live';
        // updateStrategy = 'once';
        _this.filter = new ActionsFilter();
        _this.api$ = function (query) { return api.summary(query); };
        _this.dateRange = dateRange;
        _this.store = store;
        _this.dateParam = dateParam;
        _this.query$ = _this.store.select(getActionsListQuery);
        _this.active$ = _this.store.select(getActionsSummaryActive);
        _this.data$ = _this.store.select(getActionsSummary);
        _this.summaryChart$ = _this.getSummaryChart();
        _this.loading$ = _this.store.select(getActionsSummaryLoading);
        return _this;
    }
    ActionsSummary.prototype.setActive = function (isActive) {
        if (isActive === void 0) { isActive = true; }
        isActive = isActive ? new Date().toISOString() : isActive;
        this.store.dispatch(new SetSummaryActive$1(isActive));
        if (!isActive)
            this.setLoading(false);
    };
    ActionsSummary.prototype.setLoading = function (loading) {
        this.store.dispatch(new SetSummaryLoading$1(loading));
    };
    ActionsSummary.prototype.setData = function (data) {
        this.store.dispatch(new SetSummary(data));
    };
    ActionsSummary.prototype.setQuery = function (query) {
        this.store.dispatch(new SetListQuery$1(query));
    };
    ActionsSummary.prototype.getSummaryChart = function () {
        var _this = this;
        return this.data$.pipe(map$1(function (summaryData) {
            return _this.filter.summaryCharts(_this.filter.statusQueryArray, summaryData);
        }));
    };
    ActionsSummary.prototype.clearData = function () {
        this.setActive(false);
        this.setData(null);
    };
    return ActionsSummary;
}(EntityListClient));
var ActionsSummaryClient = clientSubMixin(getQueryDataMixin(getFirstDataMixin(listQueryMixin(ActionsSummary))));

var ActionsHeatmap = /** @class */ (function () {
    function ActionsHeatmap(_a) {
        var dateRange = _a.dateRange, dateParam = _a.dateParam, api = _a.api;
        this.queryBehaviour$ = new BehaviorSubject({});
        this.query$ = this.queryBehaviour$.asObservable();
        this.loadingState$ = new BehaviorSubject(false);
        this.loading$ = this.loadingState$.asObservable();
        this.dataState$ = new BehaviorSubject(null);
        this.data$ = this.dataState$.asObservable();
        this.active$ = new BehaviorSubject(false);
        this.dataArray$ = this.data$.pipe(PageResults$);
        this.api$ = function (query) { return api.allPages(api.heatmap(query)); };
        this.dateRange = dateRange;
        this.dateParam = dateParam;
    }
    ActionsHeatmap.prototype.setActive = function (active) {
        if (active === void 0) { active = true; }
        this.active$.next(active);
        if (!active)
            this.setLoading(false);
    };
    ActionsHeatmap.prototype.setQuery = function (query) {
        if (query === void 0) { query = {}; }
        this.queryBehaviour$.next(query);
    };
    ActionsHeatmap.prototype.getDefaultQuery = function () {
        return { page_size: 300 };
    };
    ActionsHeatmap.prototype.firstDataEffect = function (data) {
        if ((data && !data.next) || !data) {
            this.setLoading(false);
        }
    };
    ActionsHeatmap.prototype.setLoading = function (isLoading) {
        this.loadingState$.next(isLoading);
    };
    ActionsHeatmap.prototype.setData = function (data) {
        this.dataState$.next(data);
    };
    ActionsHeatmap.prototype.addData = function (data) {
        this.dataState$.next(data);
    };
    ActionsHeatmap.prototype.clearData = function () {
        this.setActive(false);
        this.setData(null);
    };
    return ActionsHeatmap;
}());
var ActionsHeatmapClient = listAllClientSubMixin(getAllPageDataMixin(getFirstDataMixin(listQueryMixin(ActionsHeatmap))));

var __extends$23 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ActionsIndexAll = /** @class */ (function (_super) {
    __extends$23(ActionsIndexAll, _super);
    function ActionsIndexAll(_a) {
        var dateRange = _a.dateRange, store = _a.store, dateParam = _a.dateParam, api = _a.api;
        var _this = _super.call(this) || this;
        _this.store = store;
        _this.api$ = function (query) { return api.allPages(api.index(query)); };
        _this.dateRange = dateRange;
        _this.dateParam = dateParam;
        _this.query$ = _this.store.select(getActionsListQuery);
        _this.dataEntities$ = _this.store.select(getActionsListAllFiltered);
        _this.active$ = _this.store.select(getActionsListAllActive);
        _this.loading$ = _this.store.select(getActionsListAllLoading);
        return _this;
        // this.data$ = combineLatest(
        //   this.dataBehaviour$.asObservable(),
        //   this.filter$
        // ).pipe(
        //   map(([allData, filterFun]) => {
        //     console.log("alldata", allData);
        //     return filterFun && allData ? filterFun(allData) : allData
        //   })
        // )
    }
    ActionsIndexAll.prototype.setActive = function (isActive) {
        if (isActive === void 0) { isActive = true; }
        this.store.dispatch(new SetListAllActive(isActive));
        if (!isActive)
            this.setLoading(false);
    };
    ActionsIndexAll.prototype.setQuery = function (query) {
        this.store.dispatch(new SetListQuery$1(query));
    };
    ActionsIndexAll.prototype.addData = function (data) {
        this.store.dispatch(new AddListAll(data));
    };
    ActionsIndexAll.prototype.setData = function (data) {
        this.store.dispatch(new SetListAll(data));
    };
    ActionsIndexAll.prototype.setLoading = function (loading) {
        this.store.dispatch(new SetListAllLoading(loading));
    };
    ActionsIndexAll.prototype.clearQueryKey = function (key) {
        this.store.dispatch(new ClearQueryKey$1(key));
    };
    ActionsIndexAll.prototype.setDataMap = function (mapFunc) {
        this.store.dispatch(new SetListAllDataMap(mapFunc));
    };
    ActionsIndexAll.prototype.clearData = function () {
        this.setActive(false);
        this.setData(null);
    };
    return ActionsIndexAll;
}(EntityAllItemsClient));
var ActionsIndexAllClient = listAllClientSubMixin(getAllPageDataMixin(getFirstDataMixin(listQueryMixin(ActionsIndexAll))));

var __assign$18 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var HtActionsClient = /** @class */ (function () {
    function HtActionsClient(config) {
        this.filters = new ActionsFilter();
        var api = htClientService.getInstance().api.actions;
        this.api = api;
        var store = ApiStoreService.getNewInstance();
        store.addReducer("actions", actionsReducer);
        this.store = store;
        var dateRange = config.dateRange;
        var dateParam = 'created_at';
        this.graph = new ActionsGraphClient({ dateRange: dateRange, dateParam: dateParam, api: api, store: store });
        this.list = new ActionsListClient({ dateRange: dateRange, store: store, dateParam: dateParam, api: api });
        this.listAll = new ActionsIndexAllClient({ dateRange: dateRange, dateParam: dateParam, api: api, store: store });
        this.summary = new ActionsSummaryClient({ dateRange: dateRange, store: store, dateParam: dateParam, api: api });
        this.heatmap = new ActionsHeatmapClient({ dateRange: dateRange, dateParam: 'completed_at', api: api, store: store });
        this.initEffects();
    }
    HtActionsClient.prototype.initEffects = function () {
        var _this = this;
        this.list.query$.pipe(filter$1(function (data) { return !!data; })).subscribe(function (query) {
            _this.setListAllFilter(query);
        });
        // this.listAll.active$.pipe(filter(data => !!data)).flatMap(() => {
        //   return this.listStatusChart$()
        // })
        //   .takeUntil(this.listAll.active$.filter(data => !data).skip(1))
        //   .withLatestFrom(this.list.query$)
        //   .switchMap(([statusOverview, query]) => {
        //     // return Observable.of({})
        //     console.log(statusOverview, query);
        //     return this.getListAllUpdateQuery$(statusOverview, query)
        //   })//todo finish this
    };
    HtActionsClient.prototype.setListAllFilter = function (query) {
        var statusString = query["status"];
        var search = query["search"];
        var ids = query["id"];
        var userMarkerFilters = [];
        if (statusString) {
            var statusArray = statusString.split(",");
            // this.updateUserMap(query);
            var statusFilter_1 = [];
            statusArray.forEach(function (status) {
                statusFilter_1.push(htAction().getMarkerFilter(status));
            });
            var allStatusFilter = function (user) {
                return statusFilter_1.reduce(function (acc, filter$$1) {
                    return acc || filter$$1(user);
                }, false);
            };
            userMarkerFilters.push(allStatusFilter);
        }
        if (search) {
            userMarkerFilters.push(function (user) {
                return htAction().getMarkerSearched(search)(user); // || userMarkerFilter(user)
            });
            // this.updateUserMap(query);
        }
        if (ids) {
            var userIds_1 = ids.split(",");
            userMarkerFilters.push(function (user) {
                return userIds_1.includes(user.id);
            });
        }
        var userMarkerFilter = function (user) {
            return userMarkerFilters.reduce(function (acc, filter$$1) {
                return acc && filter$$1(user);
            }, true);
        };
        var dataMap = function (allResults) {
            var results = filter(allResults.resultsEntity, userMarkerFilter);
            var resultsEntity = indexBy(results, "id");
            return __assign$18({}, allResults, { resultsEntity: resultsEntity });
        };
        this.listAll.setDataMap(dataMap);
    };
    Object.defineProperty(HtActionsClient.prototype, "queryLabel$", {
        get: function () {
            var _this = this;
            var query$ = this.list.getApiQuery$().pipe(filter$1(function (data) { return !!data; }));
            return query$.pipe(map$1(function (query) {
                // console.log("cl", query);
                var queryLabel = _this.filters.getQueryLabel(query);
                return queryLabel;
            }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HtActionsClient.prototype, "ordering$", {
        get: function () {
            var _this = this;
            return this.list.getApiQuery$().pipe(filter$1(function (data) { return !!data; }), map$1(function (query) {
                var ordering = query ? query["ordering"] : null;
                var orderingMod = _this.getOrderingMod(ordering);
                return {
                    string: _this.filters.sortingQueryMap[orderingMod.string],
                    sign: orderingMod.sign
                };
            }), distinctUntilChanged());
        },
        enumerable: true,
        configurable: true
    });
    HtActionsClient.prototype.getOrderingMod = function (ordering) {
        var string = ordering;
        var sign = 1;
        if (ordering.includes("-")) {
            string = ordering.substring(1);
            sign = 0;
        }
        return {
            string: string,
            sign: sign
        };
    };
    return HtActionsClient;
}());
var actionsClientFactory = function (options) {
    if (options === void 0) { options = {}; }
    var dateRange = options.dateRange || dateRangeService.getInstance();
    return new HtActionsClient({ dateRange: dateRange });
};

var SET_USER_ANALYTICS = "[LOADING] set user analytics";
var SET_USER_INDEX = "[LOADING] set user index";
var SET_USER_ANALYTICS_ALL = "[LOADING] set user analytics all";
var SET_USER_INDEX_ALL = "[LOADING] set user index all";
var SetLoadingUserAnalytics = /** @class */ (function () {
    function SetLoadingUserAnalytics(payload) {
        this.payload = payload;
        this.type = SET_USER_ANALYTICS;
    }
    return SetLoadingUserAnalytics;
}());
var SetLoadingUserIndex = /** @class */ (function () {
    function SetLoadingUserIndex(payload) {
        this.payload = payload;
        this.type = SET_USER_INDEX;
    }
    return SetLoadingUserIndex;
}());
var SetLoadingUserAnalyticsAll = /** @class */ (function () {
    function SetLoadingUserAnalyticsAll(payload) {
        this.payload = payload;
        this.type = SET_USER_ANALYTICS_ALL;
    }
    return SetLoadingUserAnalyticsAll;
}());
var SetLoadingUserIndexAll = /** @class */ (function () {
    function SetLoadingUserIndexAll(payload) {
        this.payload = payload;
        this.type = SET_USER_INDEX_ALL;
    }
    return SetLoadingUserIndexAll;
}());

var LoadingDispatcher = /*#__PURE__*/Object.freeze({
    SET_USER_ANALYTICS: SET_USER_ANALYTICS,
    SET_USER_INDEX: SET_USER_INDEX,
    SET_USER_ANALYTICS_ALL: SET_USER_ANALYTICS_ALL,
    SET_USER_INDEX_ALL: SET_USER_INDEX_ALL,
    SetLoadingUserAnalytics: SetLoadingUserAnalytics,
    SetLoadingUserIndex: SetLoadingUserIndex,
    SetLoadingUserAnalyticsAll: SetLoadingUserAnalyticsAll,
    SetLoadingUserIndexAll: SetLoadingUserIndexAll
});

var SET_DATE_RANGE = "[QUERY] set date range";
var SET_USER_QUERY = "[QUERY] set user query";
var SET_USER_QUERY_RESET_PAGE = "[QUERY] set user query reset page";
var CLEAR_USER_QUERY_KEY = "[QUERY] clear user query key";
var SET_USER_ID$1 = "[USERS] set user id";
var TOGGLE_USER_ID = "[USERS] toggle user id";
var SET_PLACELINE_QUERY$1 = "[USERS] Set placeline query";
var SetDateRange = /** @class */ (function () {
    function SetDateRange(payload) {
        this.payload = payload;
        this.type = SET_DATE_RANGE;
    }
    return SetDateRange;
}());
var SetUserQuery = /** @class */ (function () {
    function SetUserQuery(payload) {
        this.payload = payload;
        this.type = SET_USER_QUERY;
    }
    return SetUserQuery;
}());
var SetUserQueryResetPage = /** @class */ (function () {
    function SetUserQueryResetPage(payload) {
        this.payload = payload;
        this.type = SET_USER_QUERY_RESET_PAGE;
    }
    return SetUserQueryResetPage;
}());
var ClearUserQueryKey = /** @class */ (function () {
    function ClearUserQueryKey(payload) {
        this.payload = payload;
        this.type = CLEAR_USER_QUERY_KEY;
    }
    return ClearUserQueryKey;
}());
var SetUserId$1 = /** @class */ (function () {
    function SetUserId(payload) {
        this.payload = payload;
        this.type = SET_USER_ID$1;
    }
    return SetUserId;
}());
var ToggleUserId = /** @class */ (function () {
    function ToggleUserId(payload) {
        this.payload = payload;
        this.type = TOGGLE_USER_ID;
    }
    return ToggleUserId;
}());
var SetPlacelineQuery$1 = /** @class */ (function () {
    function SetPlacelineQuery(payload) {
        if (payload === void 0) { payload = {}; }
        this.payload = payload;
        this.type = SET_PLACELINE_QUERY$1;
    }
    return SetPlacelineQuery;
}());

var QueryDispatcher = /*#__PURE__*/Object.freeze({
    SET_DATE_RANGE: SET_DATE_RANGE,
    SET_USER_QUERY: SET_USER_QUERY,
    SET_USER_QUERY_RESET_PAGE: SET_USER_QUERY_RESET_PAGE,
    CLEAR_USER_QUERY_KEY: CLEAR_USER_QUERY_KEY,
    SET_USER_ID: SET_USER_ID$1,
    TOGGLE_USER_ID: TOGGLE_USER_ID,
    SET_PLACELINE_QUERY: SET_PLACELINE_QUERY$1,
    SetDateRange: SetDateRange,
    SetUserQuery: SetUserQuery,
    SetUserQueryResetPage: SetUserQueryResetPage,
    ClearUserQueryKey: ClearUserQueryKey,
    SetUserId: SetUserId$1,
    ToggleUserId: ToggleUserId,
    SetPlacelineQuery: SetPlacelineQuery$1
});

var Dispatcher$1 = {
    groups: GroupsDispatcher,
    users: UsersDispatcher,
    loading: LoadingDispatcher,
    query: QueryDispatcher,
    Segment: SegmentsDispatcher
};

//helper
// export * from "./helpers/operators"
// import "rxjs/add/operator/let";

export { defaultListConfig, ApiType, HtUsersClient, usersClientFactory, UsersPlaceline, UsersPlacelineClient, UsersAnalytics, UsersAnalyticsClient, UsersAnalyticsListAll, UsersAnalyticsListAllClient, UsersSummary, UsersSummaryClient, UsersHeatmap, UsersHeatmapClient, DefaultUsersFilter, ActionsFilter, HtGroupsClient, groupsClientFactory, GroupsList, GroupsListClient, GroupsItem, GroupsItemClient, AccountsClient, AccountUser, AccountUserClient, MembershipsAll, MemberShipsClient, HtActionsClient, actionsClientFactory, ActionsGraph, ActionsGraphClient, ActionsList, ActionsListClient, ActionsSummary, ActionsSummaryClient, ActionsHeatmap, ActionsHeatmapClient, defaultDateRange, DateRange, dateRangeFactory, dateRangeService, HtClient, initClient, htClientFactory, htClientService, reducers, reducer$1 as reducer, metaReducers, getUsersState, getUsersUsersData, getUsersPlacelineId, getUsersPlacelineQuery, getUsersPlacelineLoading, getUsersListActive, getUsersListId, getUsersListQuery, getUsersAnalyticsPage, getUsersAnalyticsLoading, getUsersIndexPage, getUsersIndexIsActive, getUsersAnalyticsIsActive, getUsersIndexMarkersIsActive, getUsersAnalyticsMarkersIsActive, getUsersAnalyticsAllLoading, getUsersIndexFilteredMarker, getUsersAnalyticsFilteredMarker, getUsersIndexAll, getUsersAnalyticsAll, getUsersSummary, getUsersSummaryActive, getUsersSummaryLoading, getSegmentsState, getSegmentsSelectedId, getSegmentsResetMapId, getGroupsState, getGroupId, getGroupAll, getGroupListActive, getAccountState, getAccountUser$1 as getAccountUser, getAccountMembershipsAll, getAccountKey, getAccountTempKey, getAccountCurrentKey, getAccountUserId, getActionsState, getActionsList, getActionsListLoading, getActionsListActive, getActionsListQuery, getActionsListAll, getActionsListAllActive, getActionsListAllLoading, getActionsListAllFiltered, getActionsSummary, getActionsSummaryActive, getActionsSummaryLoading, getActionsSummaryQuery, getActionsGraph, getActionsGraphLoading, getActionsGraphActive, getActionsGraphQuery, Dispatcher$1 as Dispatcher };
