import {listQueryMixin} from "../../mixins/entity-query";
import {listAllClientSubMixin} from "../../mixins/list-all-client-sub";
import {getAllPageDataMixin} from "../../mixins/get-data";
import {getFirstDataMixin} from "../../mixins/get-first-data";
import {Observable, Subscription, of, BehaviorSubject} from "rxjs";
import {IPageClientConfig} from "../../interfaces";
import {IActionHeat, Page, IPlaceHeat} from "ht-models";
import {HtApi, HtUsersApi} from "ht-api";
import {IAllowedQueryMap, PageResults$} from "ht-data";
import {DateRange} from "../../global/date-range";

export class UsersHeatmap {
  query$: Observable<object> = of({});
  api$: (query) => Observable<Page<IPlaceHeat>>;
  loadingState$ = new BehaviorSubject(false);
  loading$ = this.loadingState$.asObservable();
  dataState$: BehaviorSubject<Page<IActionHeat> | null> = new BehaviorSubject(null);
  data$ = this.dataState$.asObservable();
  dataSub: Subscription;
  dateRange;
  active$: BehaviorSubject<boolean> = new BehaviorSubject(false);
  dataArray$ = this.data$.pipe(PageResults$);
  dateParam: string;
  constructor({ dateRange, dateParam, api }: IPageClientConfig<HtUsersApi>) {
    this.api$ = query => api.allPages(api.heatmap(query));
    this.dateRange = dateRange;
    this.dateParam = dateParam;
  }

  setActive(active = true) {
    this.active$.next(active);
    if (!active) this.setLoading(false)
  }

  getDefaultQuery() {
    return {page_size: 300}
  };

  firstDataEffect(data) {
    if ((data && !data.next) || !data) {
      this.setLoading(false);
    }
  }

  setLoading(isLoading: boolean) {
    this.loadingState$.next(isLoading)
  }

  setData(data: Page<IActionHeat>) {
    this.dataState$.next(data)
  }

  addData(data) {
    this.dataState$.next(data)
  }

  clearData() {
    // this.setLoading(false);
    this.setActive(false);
    this.setData(null);
  }
};

export const UsersHeatmapClient = listAllClientSubMixin(
  getAllPageDataMixin(getFirstDataMixin(listQueryMixin(UsersHeatmap)))
);