import {combineLatest, merge,  Observable ,  Subscription } from "rxjs";
import * as fromRoot from "../../reducers";
import * as fromSegmentsDispatcher from "../../dispatchers/segments-dispatcher";
import * as fromUsersDispatcher from "../../dispatchers/user-dispatcher";
import { EntityItemClient } from "../../base/item-client";
import { getIdQueryDataMixin } from "../../mixins/get-data";
import { itemQueryMixin } from "../../mixins/entity-query";
import { clientSubMixin } from "../../mixins/client-subscription";
import { IClientConfig } from "../../interfaces";
import { IUserPlaceline } from "ht-models";
import { getFirstDataMixin } from "../../mixins/get-first-data";
import {HtUsersApi} from "ht-api";
import {distinctUntilChanged, map} from "rxjs/operators";

export class UsersPlaceline extends EntityItemClient {
  name = "users placeline";
  updateStrategy = "live";
  api$: (id, query) => Observable<IUserPlaceline>;
  store;
  data$;
  loading$;
  segmentsState$;
  segmentSelectedId$;
  segmentResetId$;
  actionId$;
  hasData$;
  constructor({ store, api }: IClientConfig<HtUsersApi>) {
    super();
    this.api$ = (id, query) => api.placeline(id, query);
    this.store = store;
    this.query$ = this.store.select(fromRoot.getUsersPlacelineQuery);
    this.actionId$ = this.getQueryType('action_id');
    this.data$ = this.store.select(fromRoot.getUsersUsersData);
    this.loading$ = this.store.select(fromRoot.getUsersPlacelineLoading);
    this.id$ = this.store.select(fromRoot.getUsersPlacelineId);
    this.segmentsState$ = this.store.select(fromRoot.getSegmentsState);
    this.segmentSelectedId$ = this.store.select(fromRoot.getSegmentsSelectedId);
    this.segmentResetId$ = this.store.select(fromRoot.getSegmentsResetMapId);
    this.hasData$ = combineLatest(
      this.id$,
      this.query$,
    ).pipe(
      map(([id, query]) => {
        // console.log("here", id, query, this.validQueryData([id, query || {}]));
        return !!this.validQueryData([id, query || {}])
      }),
      distinctUntilChanged()
      // share
    );
    // this.init()
  };

  getQueryType(type: string) {
    return this.query$.pipe(
      map(query => {
        return query ? query[type] : null
      }),
      distinctUntilChanged()
    )
  }

  setData(data) {
    this.store.dispatch(new fromUsersDispatcher.SetUserData(data));
  }
  setLoading(data) {
    this.store.dispatch(new fromUsersDispatcher.SetPlacelineLoading(data));
  }
  setId(id) {
    this.store.dispatch(new fromUsersDispatcher.SetPlacelineId(id));
    if (!id) this.setLoading(false);
  }
  toggleId(userId: string) {
    this.store.dispatch(new fromUsersDispatcher.TogglePlacelineId(userId));
  }
  setQuery(query) {
    this.store.dispatch(new fromUsersDispatcher.SetPlacelineQuery(query));
  }
  setSegmentSelectedId(segmentId) {
    this.store.dispatch(new fromSegmentsDispatcher.SetSelectedId(segmentId));
  }
  setSegmentResetMapId(segmentId: string) {
    this.store.dispatch(new fromSegmentsDispatcher.SetResetMapId(segmentId));
  }

  getMapData$() {
    // return dataWithSelectedId$(this.data$, this.segmentSelectedId$, [
    //   "segments"
    // ]);
  }

  validQueryData([id, query]): boolean {
    return !!id || query.action_id || query.action_unique_id
  }

  clearData() {
    this.setData(null);
    this.setLoading(false);
    this.setQuery({});
    this.setId(null)
  }
}

export const UsersPlacelineClient = clientSubMixin(
  getIdQueryDataMixin(getFirstDataMixin(itemQueryMixin(UsersPlaceline)))
);

